
voorspel_lgc <- function(mod, df, aantal_dagen = 7, level_pi = .80) {
  # Haal de naam van de afhankelijke variabele op
  dep_naam <- all.vars(summary(mod)$formula)[1]
  
  # Maak een nieuw data frame
  num <- 1:(nrow(df) + aantal_dagen)
  datum_nieuw <- map_dfr(list(x = 1:aantal_dagen), ~ max(df$datum) + days(.))
  datum <- c(df$datum, datum_nieuw)
  df_pred <- tibble(datum, num)
  
  # Voeg de afhankelijke variabele toe
  dep <- c(pull(df, dep_naam), rep(NA, aantal_dagen))
  df_pred[dep_naam] <- dep
  
  # Maak voorspellingen en transformeer ze terug
  # N.B.: Ik heb hier voor predictie-intervallen gekozen, dat lijkt me in dat geval geschikter dan
  # betrouwbaarheidsintervallen. Eens?
  pred <- predFit(mod, df_pred, interval = "prediction", level = level_pi) %>%
    as_tibble()
  pred$datum <- df_pred$datum
  pred$obs <- pull(df_pred, dep_naam)
  
  return(pred)
}


muteer_df_nieuwe_opnames <- function(df_nieuwe_opnames) {
  df <- df_nieuwe_opnames %>%
    mutate(num = 1:nrow(.))

  df$weekend <- ifelse(weekdays(df$datum) %in% c("zaterdag", "zondag"), 1, 0)

  return(df)
}

voorspel_instroom = function(df_nieuwe_opnames, aantal_dagen, scenarios, voorspeldatum, print_ = T, exclude_until = as.Date('0000-01-01')){
  #aantal dagen dat instroom vooruit voorspelt moet worden > aantal dagen dat de bezetting vooruit voorspelt moet worden
  aantal_dagen = max(0, as.integer(as.Date(voorspeldatum) - max(df_nieuwe_opnames$datum))) + aantal_dagen
  
  df_nieuwe_opnames = muteer_df_nieuwe_opnames(df_nieuwe_opnames)
  datum_nieuw <- map_dfr(list(x = 1:aantal_dagen), ~ max(df_nieuwe_opnames$datum) + days(.))
  nieuwe_opnames = tibble(datum = c(df_nieuwe_opnames$datum, datum_nieuw))
  
  # Richards curve ------------------------------------------------------------------------------
  if('log' %in% scenarios | 'log_upr' %in% scenarios | 'log_lwr' %in% scenarios){
    df_nieuwe_opnames_RC = df_nieuwe_opnames %>% filter(datum >= exclude_until)
    df_nieuwe_opnames_RC$aantal_cumulatief = cumsum(df_nieuwe_opnames_RC$aantal)
   
    # Fit een 4 of 5-parameter model, afhankelijk van hoeveel parameters modpar geschat
    # heeft
    nieuwe_opnames = tryCatch({
      # Schat parameters
      model_params <- modpar(df_nieuwe_opnames_RC$num, df_nieuwe_opnames_RC$aantal_cumulatief,
                             # force4par = TRUE, # Alleen parameters voor de Richards curve
                             pn.options = "myoptions", suppress.text = TRUE)
      if (is.na(model_params$RAsym)) {
        lgc <- nlsLM(aantal_cumulatief ~
                     SSposnegRichards(num,
                                      Asym = Asym, K = K, Infl = Infl, M = M,
                                      modno = 12, # Model 12 is een simpele Richards curve met 4 parameters
                                      pn.options = "myoptions"
                     ),
                   data = df_nieuwe_opnames_RC, control = nls.control(maxiter = 1000, minFactor = 1 / 4096 / 40)
        )
      } else {
        lgc <- nlsLM(aantal_cumulatief ~
                     SSposnegRichards(num,
                                      Asym = Asym, K = K, Infl = Infl, M = M, RAsym = RAsym,
                                      modno = 11, # Model 11 is een Richards curve met 5 parameters
                                      pn.options = "myoptions"
                     ),
                   data = df_nieuwe_opnames_RC, control = nls.control(maxiter = 1000, minFactor = 1 / 4096 / 40)
        )
              }
  
  
        preds_lgc <- voorspel_lgc(lgc, df_nieuwe_opnames, aantal_dagen = aantal_dagen) %>% mutate(fit = ifelse(datum < exclude_until, NA, fit), 
                                                                                                  lwr = ifelse(datum < exclude_until, NA, lwr),
                                                                                                  upr = ifelse(datum < exclude_until, NA, upr))
        voorspel_nieuwe_opnames(preds_lgc)
      },
      error=function(cond) {
        # Dit wordt alleen gebruikt als SSposnegRichards het niet doet (bij erg weinig data, begin 2e golf)
        # In dit geval fitten we direct de intensiteit van de richards curve op de nieuwe aankomsten met Poisson regressie
        RC = Poisregr_logistic(df_nieuwe_opnames_RC$datum, df_nieuwe_opnames_RC$aantal)
        
        # opslaan in voorspeltabel
        nieuwe_opnames$obs = c(df_nieuwe_opnames$aantal_cumulatief, rep(NA, aantal_dagen))
        nieuwe_opnames$nieuwe_opn = c(df_nieuwe_opnames$aantal, rep(NA, aantal_dagen))
        nieuwe_opnames$log = RC(nieuwe_opnames$datum)
        
        nieuwe_opnames$fit = cumsum(nieuwe_opnames$log)
        nieuwe_opnames$log_lwr = qpois(0.1, RC(nieuwe_opnames$datum))
        nieuwe_opnames$log_upr = qpois(0.9, RC(nieuwe_opnames$datum))
        nieuwe_opnames$upr = cumsum(nieuwe_opnames$log_upr)
        nieuwe_opnames$lwr = cumsum(nieuwe_opnames$log_lwr)
        nieuwe_opnames
    })
    }
  
  if('vlak' %in% scenarios){
  # Vlak gemiddelde -----------------------------------------------------------------------------
  
  # Gemiddelde instroom laatste zeven dagen?
  gem_aantal_opnames <- nieuwe_opnames %>%
    drop_na(nieuwe_opn) %>%
    tail(7) %>%
    pull(nieuwe_opn) %>%
    mean()
  
  pred_vlak <- rep(gem_aantal_opnames, aantal_dagen)
  
  nieuwe_opnames$vlak <- c(rep(NA, nrow(nieuwe_opnames) - aantal_dagen), pred_vlak)}
  
  if('piek' %in% scenarios){
  # Nieuwe Richards curve -----------------------------------------------------------
  
  laatste_pred_nieuwe_opnames <- nieuwe_opnames %>% 
    filter(datum == max(df_nieuwe_opnames$datum)) %>% 
    pull(log)
  
  startpunt <- which(nieuwe_opnames$log > laatste_pred_nieuwe_opnames)[1]
  
  # Maak nieuwe voorspellingen door dezelfde Richards curve te fitten vanaf X dagen terug
  # (X = startpunt)
  pred_piek <- voorspel_lgc(lgc,  tail(df_nieuwe_opnames, startpunt), aantal_dagen) %>% 
    voorspel_nieuwe_opnames() %>% 
    pull(log) %>% 
    tail(aantal_dagen + 1)
  
  nieuwe_opnames$piek <- c(
    rep(NA, nrow(nieuwe_opnames) - aantal_dagen - 1),
    pred_piek)}
  
  if('exp' %in% scenarios){
    # Exponentieel --------------------------------------------------------------------------------
    
    pred_exp <- map_dbl(1:aantal_dagen, ~ gem_aantal_opnames * (2^(1 / 9))^.x)
    
    nieuwe_opnames$exp <- c(rep(NA, nrow(nieuwe_opnames) - aantal_dagen), pred_exp)
  }
  voorspelde_opnames = nieuwe_opnames
  return(voorspelde_opnames)


}