# Kaplan-Meier ---------------------------------------------------------------------

fit_kaplan_meier <- function(data) {
  surv_object <- Surv(time = data$ligduur, event = data$event)
  fit1 <- surv_fit(surv_object ~ 1, data = data, conf.type = "plain")
  
  return(fit1)
}

survival_analyse = function(ziekenhuis, ICset, Kliniekset){
  
  # Data inladen  ---------------------------------------------------------------------
  
  # ligduur 1 = dezelfde dag onstslag; ligduur 2 = de volgende dag -> er is geen ligduur00
  # meer. code aangepast hiervoor
  
  df_kl_ic <- Kliniekset
  df_ic_kl <- ICset
  
  waarde_vertrek_ic_zkh <- "IC - ander ziekenhuis"
  waarde_vertrek_kl_zkh <- "Kliniek - ander ziekenhuis"
  waarde_vertrek_kl_ic <- "Kliniek - vertrek naar IC"
  waarde_vertrek_ic_kl <- "IC - vertrek naar Kliniek"

  df_ic_kl$event <- ifelse(df_ic_kl$vertrek_type == waarde_vertrek_ic_kl, 1, 0)
  df_kl_ic$event <- ifelse(df_kl_ic$vertrek_type == waarde_vertrek_kl_ic, 1, 0)
  
  # empirische kansverdeling -> kaplan meier werkt niet goed met veel gecensoreerde data
  if(any(df_kl_ic$event == 1)){df_kl_ic <- filter(df_kl_ic, event == 1)}
  if(any(df_ic_kl$event == 1)){df_ic_kl <- filter(df_ic_kl, event == 1)}
  df_kl_ic_gbhbp <- filter(df_kl_ic, BehandelBeperking == 'Geen behandelbeperking') #ook uitzondering maken als geen bhbp
  
  df_kl_zonder_overplaatsingen <- filter(Kliniekset, !vertrek_type == waarde_vertrek_kl_zkh)
  df_kl_zonder_overplaatsingen_gbhbp <- filter(Kliniekset, !vertrek_type == waarde_vertrek_kl_zkh) %>% filter(BehandelBeperking == 'Geen behandelbeperking')
  df_ic_zonder_overplaatsingen <- filter(ICset, !vertrek_type == waarde_vertrek_ic_zkh)
  
  perc_ic_kl <- sum(df_ic_kl$event == 1) / nrow(df_ic_zonder_overplaatsingen)
  perc_kl_ic <- sum(df_kl_ic$event == 1) / nrow(df_kl_zonder_overplaatsingen)
  perc_kl_ic_gbhbp <- sum(df_kl_ic_gbhbp$event == 1) / nrow(df_kl_zonder_overplaatsingen_gbhbp)
  
  fit_kl <- fit_kaplan_meier(Kliniekset)
  fit_ic <- fit_kaplan_meier(ICset)
  fit_kl_ic <- fit_kaplan_meier(df_kl_ic)
  fit_kl_ic_gbhbp = fit_kaplan_meier(df_kl_ic_gbhbp) #fit als geen bhbp
  fit_ic_kl <- fit_kaplan_meier(df_ic_kl)
  
  # Kansverdeling kliniek ------------------------------------------------------------
  
  condprobs_kl <- est_condprobs(fit_kl)
  
  # Kansverdeling IC -----------------------------------------------------------------
  
  condprobs_ic <- est_condprobs(fit_ic, max_ligduur = 90)
  
  # Kansverdeling kliniek naar IC ---------------------------------------------------
  
  condprobs_kl_ic <- est_condprobs_doorstroom(fit_kl_ic, fit_kl, perc = perc_kl_ic)
  
  # Kansverdeling kliniek naar IC als geen bhbp---------------------------------------------------
  
  condprobs_kl_ic_gbhbp <- est_condprobs_doorstroom(fit_kl_ic_gbhbp, fit_ic, perc = perc_kl_ic_gbhbp)
  
  # Kansverdeling IC naar kliniek ---------------------------------------------------
  
  condprobs_ic_kl <- est_condprobs_doorstroom(fit_ic_kl, fit_ic, perc = perc_ic_kl)
  
  return(mget(c('fit_kl', 'fit_ic', 'fit_kl_ic', 'fit_ic_kl','fit_kl_ic_gbhbp',
   'condprobs_kl','condprobs_ic','condprobs_ic_kl' , 'condprobs_kl_ic', 'condprobs_kl_ic_gbhbp')))
}




