Sys.setlocale("LC_ALL", "English")
rm(list=ls())
source('R/utils.R')
source('R/00_maak_datasets.R')
require(reshape2)
require(tibbletime)
library(xtable)
library(scales)
library(tables)


rep.col<-function(x,n){
  matrix(rep(x,each=n), ncol=n, byrow=TRUE)
}


maak_voorspeltabel_det = function(summaries, Kliniekset, ICset, scenarios, horizons ,overplaatsingen_er_uit, save_plots = T,
                                            max_date = as.Date('2020-07-10')){
  #haal voorspellingen op + boven/ondergrens
  
  horizons = c(0, horizons) #voeg horizon 0 toe om huidige bezetting uit te rekenen
  for(i in seq(length(horizons))){
    horizon = horizons[i]
    preds_ic = c()
    preds_kl = c()
    dates = c()
    for(datestring in names(summaries)){
      if(as.Date(datestring) <= max_date){
        df_ic = summaries[[datestring]]$preds_bezet_tot_ic %>% select(datum, scenarios)
        row_ic = as.numeric(df_ic[df_ic$datum == as.Date(datestring) + days(horizon), ] %>% select(-datum))
        preds_ic = rbind(preds_ic, row_ic)
        
        df_kl = summaries[[datestring]]$preds_bezet_tot_kl %>% select(datum, scenarios)
        row_kl = as.numeric(df_kl[df_kl$datum == as.Date(datestring) + days(horizon), ] %>% select(-datum))
        preds_kl = rbind(preds_kl, row_kl)
        
        dates = c(dates, toString(as.Date(datestring) + days(horizon)))}
    }
    
    colnames(preds_ic) = sapply(colnames(df_ic %>% select(-datum)), function(str){paste('Predicted occupancy ICU (', horizon, ifelse(horizon>1,' days ahead)', ' day ahead)'))})
    colnames(preds_kl) = sapply(colnames(df_kl %>% select(-datum)), function(str){paste('Predicted occupancy Ward (', horizon, ifelse(horizon>1,' days ahead)', ' day ahead)'))})
    
    
    df_ic = as_tibble(preds_ic) %>% add_column(date = as.Date(dates), .before = 1)
    df_kl = as_tibble(preds_kl) %>% add_column(date = as.Date(dates), .before = 1)
    
    if(i ==1){
      pred_tbl_ic = df_ic
      pred_tbl_kl = df_kl
    }else{
      pred_tbl_ic = pred_tbl_ic %>% left_join(df_ic, by = 'date') 
      pred_tbl_kl = pred_tbl_kl %>% left_join(df_kl, by = 'date')
    }
  }
  horizons = horizons[-1] # horizon 0 weer weghalen
  
  voorspeltabel_benutting_ic = pred_tbl_ic %>%  rename(`Realised occupancy at ICU` = `Predicted occupancy ICU ( 0  day ahead)`)
  voorspeltabel_benutting_kl = pred_tbl_kl %>% rename(`Realised occupancy at Ward` = `Predicted occupancy Ward ( 0  day ahead)`)
  
  #haal fout/coverage resultaten op voor IC
  errors_ic = voorspeltabel_benutting_ic[-seq(max(horizons)),] %>% select(-date, -`Realised occupancy at ICU`) - voorspeltabel_benutting_ic[-seq(max(horizons)),rep("Realised occupancy at ICU", ncol(voorspeltabel_benutting_ic) - 2)]
  
  ic_eval = tibble(
    type = sapply(horizons, function(days){paste0('Occupancy ', days, ' days ahead')}),
    bias = apply(errors_ic,2,function(v){mean((v), na.rm=T)}),
    MAE = apply(errors_ic,2,function(v){mean(abs(v), na.rm=T)}))
  
  #haal fout/coverage resultaten op voor Kliniek
  errors_kl = voorspeltabel_benutting_kl[-seq(max(horizons)),] %>% select(-date, -`Realised occupancy at Ward`) - voorspeltabel_benutting_kl[-seq(max(horizons)),rep("Realised occupancy at Ward", ncol(voorspeltabel_benutting_kl) - 2)]
  
  kl_eval = tibble(
    type = sapply(horizons, function(days){paste0('Occupancy ', days, ' days ahead')}),
    bias = apply(errors_kl,2,function(v){mean((v), na.rm=T)}),
    MAE = apply(errors_kl,2,function(v){mean(abs(v), na.rm=T)}))
  
  #make the table long (for writing to  LaTeX)
  horizons_int = rep(horizons, each = ncol(ic_eval)-1)
  ic_eval_long = pivot_longer(ic_eval, -type) 
  kl_eval_long = pivot_longer(kl_eval, -type)
  departments   = rep(c("ICU", "Ward"), each = nrow(ic_eval_long))
  tot_eval = full_join(ic_eval_long, kl_eval_long) %>% add_column(dept = departments, .before = 1) %>% mutate(value = round(value,2))
  
  #make weekly average predictions 
  ma_pred_wk = rollify(mean, window = 7) # get weekly moving average fn
  voorspelling_ma_ic = ma_pred_wk(voorspeltabel_benutting_ic$`Realised occupancy at ICU`)
  voorspelling_ma_kl = ma_pred_wk(voorspeltabel_benutting_kl$`Realised occupancy at Ward`)
  
  
  errors_ic_ma =  rep.col(as.matrix(voorspelling_ma_ic),length(horizons)) -  sapply(horizons, function(hor){c((voorspeltabel_benutting_ic %>% pull("Realised occupancy at ICU"))[-seq(hor)], rep(NA, hor))})
  ic_eval_ma = tibble(
    type = sapply(horizons, function(days){paste0('Occupancy ', days, ' days ahead')}),
    bias = apply(errors_ic_ma,2,function(v){mean((v), na.rm=T)}),
    MAE = apply(errors_ic_ma,2,function(v){mean(abs(v), na.rm=T)}))
  
  errors_kl_ma =  rep.col(as.matrix(voorspelling_ma_kl),length(horizons)) -  sapply(horizons, function(hor){c((voorspeltabel_benutting_kl %>% pull("Realised occupancy at Ward"))[-seq(hor)], rep(NA, hor))})
  kl_eval_ma = tibble(
    type = sapply(horizons, function(days){paste0('Occupancy ', days, ' days ahead')}),
    bias = apply(errors_kl_ma,2,function(v){mean((v), na.rm=T)}),
    MAE = apply(errors_kl_ma,2,function(v){mean(abs(v), na.rm=T)}))
  
  #make the table long (for writing to  LaTeX)
  horizons_int = rep(horizons, each = ncol(ic_eval)-1)
  ic_eval_long_ma = pivot_longer(ic_eval_ma, -type) 
  kl_eval_long_ma = pivot_longer(kl_eval_ma, -type)
  departments   = rep(c("ICU", "Ward"), each = nrow(ic_eval_long_ma))
  tot_eval_ma = full_join(ic_eval_long_ma, kl_eval_long_ma) %>% add_column(dept = departments, .before = 1) %>% mutate(value = round(value,2))
  
  return(mget(c('tot_eval', 'tot_eval_ma')))
}


if(sys.nframe() == 0){
  # hier stel je param in voor plots, eerst moet "haal_voorspellingen_op.R" met deze parameters gedraaid worden en er moet dan een .rds file weggeschreven zijn met deze parameter instellingen
  ziekenhuis = 'LUMC'
  scenarios = c('log')
  horizons <- c(1,2,3,5)
  aantal_dagen = 5
  overplaatsingen_er_uit = T
  load(paste0('R/data analyse/data/rolling_preds_',ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),aantal_dagen,'_det.Rds'))
  
  source('R/00_maak_datasets.R')
  list2env(maak_datasets(ziekenhuis), envir = .GlobalEnv)
  res_tot = maak_voorspeltabel_det(summaries, Kliniekset, ICset, scenarios, horizons, overplaatsingen_er_uit)
  # 
  # tot_df_eval = full_join(res, res2)
  # results_table <- tabular(RowFactor(type, name = 'type') ~
  #                            Heading()*Factor(dept,
  #                                             levelnames =  c("ICU",
  #                                                             "Ward" ))*
  #                            Heading()*Factor(name,
  #                                             levelnames = c('bias', 'CR', 'MAE' ))*Heading()*(value)*Heading()*(mean)*
  #                            DropEmpty(empty=""),
  #                          data = tot_df_eval)
  # results_table_ma = tabular(RowFactor(type, name = 'type') ~
  #                              Heading()*Factor(dept,
  #                                               levelnames =  c("ICU",
  #                                                               "Ward" ))*
  #                              Heading()*Factor(name,
  #                                               levelnames = c('bias', 'MAE' ))*Heading()*(value)*Heading()*(mean)*
  #                              DropEmpty(empty=""),
  #                            data = res_ma)
  # write(tables::latex.tabular(results_table)[[1]], file = paste0('R/data analyse/data/foutentabel_',ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),aantal_dagen,'.tex'))
  # write(tables::latex.tabular(results_table_ma)[[1]], file = paste0('R/data analyse/data/foutentabel_ma_',ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),aantal_dagen,'.tex'))
}
