rm(list = ls())
Sys.setlocale("LC_ALL", "English")
source('R/utils.R')
source('R/00_maak_datasets.R')
require(reshape2)
require(tibbletime)
library(xtable)
library(scales)
library(tables)
integer_breaks <- function(n = 5, ...) {
  fxn <- function(x) {
    breaks <- floor(pretty(x, n, ...))
    names(breaks) <- attr(breaks, "labels")
    breaks
  }
  return(fxn)
}

rep.col<-function(x,n){
  matrix(rep(x,each=n), ncol=n, byrow=TRUE)
}
colorshorizons = c('0' = 'black','1'='orange','2'='cyan','3' = 'blue','5'='purple','7'='green' )


maak_plots_maxima = function(summaries, Kliniekset, ICset, horizon, overplaatsingen_er_uit,  taal  ='EN', save_plots = T, CI_val = 0.95, max_date = as.Date('2020-07-10')){
  preds = c()
  dates = c()
  realisatie_ic = c()
  realisatie_kl = c()
  for(datestring in names(summaries)){
    if(as.Date(datestring)<= max_date){
      dates = c(dates, toString(as.Date(datestring) + days(horizon)))
      maximum_ic = mean(summaries[[datestring]]$maxima_IC$log[horizon+1,])
      ub_maximum_ic = quantile(summaries[[datestring]]$maxima_IC$log[horizon+1,], 1-(1-CI_val)/2)
      lb_maximum_ic = quantile(summaries[[datestring]]$maxima_IC$log[horizon+1,], 1-(1+CI_val)/2)
      maxima_kl = mean(summaries[[datestring]]$maxima_kl$log[horizon+1,])
      ub_maxima_kl = quantile(summaries[[datestring]]$maxima_kl$log[horizon+1,], 1-(1-CI_val)/2)
      lb_maxima_kl = quantile(summaries[[datestring]]$maxima_kl$log[horizon+1,], 1-(1+CI_val)/2)
      preds = rbind(preds, c(maximum_ic, maxima_kl, ub_maximum_ic, ub_maxima_kl, lb_maximum_ic, lb_maxima_kl))
      realisatie_ic = c(realisatie_ic, summaries[[datestring]]$pred_df_ic %>% filter(datum == datestring) %>% pull('avg_log'))
      realisatie_kl = c(realisatie_kl, summaries[[datestring]]$pred_df_kl %>% filter(datum == datestring) %>% pull('avg_log'))
    }}
  pred_tbl = tibble(datum = as.Date(dates), `voorspelling maximum IC` = preds[,1],  `voorspelling maximum Kliniek` = preds[,2],
                    `Bovengrens voorspelling maximum IC` = preds[,3], `Bovengrens voorspelling maximum Kliniek` = preds[,4],
                    `Ondergrens voorspelling maximum IC` = preds[,5], `Ondergrens voorspelling maximum Kliniek` = preds[,6],)
  pred_tbl = pred_tbl %>% filter(datum <= max_date)
  
  historische_maximale_bezetting  = list(hist_bezetting_ic = tibble(datum = as.Date(names(summaries)[as.Date(names(summaries))<= max_date]), bezetting = realisatie_ic),
                                         hist_bezetting_kl = tibble(datum = as.Date(names(summaries)[as.Date(names(summaries))<= max_date]), bezetting = realisatie_kl))
  maxfn = rollify(max, window = horizon)
  historische_maximale_bezetting$hist_bezetting_ic = historische_maximale_bezetting$hist_bezetting_ic %>% mutate(bezetting = maxfn(bezetting))
  historische_maximale_bezetting$hist_bezetting_kl = historische_maximale_bezetting$hist_bezetting_kl %>% mutate(bezetting = maxfn(bezetting))
  
  daadwerkelijke_maxima_IC = historische_maximale_bezetting$hist_bezetting_ic%>% filter(as.Date(datum) %in% pred_tbl$datum)
  daadwerkelijke_maxima_Kliniek = historische_maximale_bezetting$hist_bezetting_kl %>% filter(as.Date(datum) %in% pred_tbl$datum)
  
  voorspeltabel_maxima_ic = pred_tbl[-seq(horizon),] %>% select(-`voorspelling maximum Kliniek`,
                                                                -`Bovengrens voorspelling maximum Kliniek`,
                                                                -`Ondergrens voorspelling maximum Kliniek`) %>% left_join( daadwerkelijke_maxima_IC, by = 'datum')%>%
    rename(`daadwerkelijk maximum` = bezetting)
  
  voorspeltabel_maxima_kl = pred_tbl[-seq(horizon),] %>% select(-`voorspelling maximum IC`,
                                                                -`Bovengrens voorspelling maximum IC`,
                                                                -`Ondergrens voorspelling maximum IC`)  %>% left_join( daadwerkelijke_maxima_Kliniek, by = 'datum') %>% 
    rename(`daadwerkelijk maximum` = bezetting)
  
  
  #haal fout/coverage resultaten op voor IC
  errors_ic = voorspeltabel_maxima_ic$`voorspelling maximum IC` - voorspeltabel_maxima_ic$`daadwerkelijk maximum`
  indc_covg_ic = voorspeltabel_maxima_ic$`Bovengrens voorspelling maximum IC` >= voorspeltabel_maxima_ic$`daadwerkelijk maximum` &voorspeltabel_maxima_ic$`Ondergrens voorspelling maximum IC` <= voorspeltabel_maxima_ic$`daadwerkelijk maximum`  
  
  ic_eval = tibble(
    type = paste0('Max. ',horizon,'d. ah.'),
    bias = mean(errors_ic, na.rm = T),
    MAE = mean(abs(errors_ic), na.rm=T),
    CR = mean(indc_covg_ic, na.rm=T))
  
  #haal fout/coverage resultaten op voor Kliniek
  errors_kl = voorspeltabel_maxima_kl$`voorspelling maximum Kliniek` - voorspeltabel_maxima_kl$`daadwerkelijk maximum`
  indc_covg_kl = voorspeltabel_maxima_kl$`Bovengrens voorspelling maximum Kliniek` > voorspeltabel_maxima_kl$`daadwerkelijk maximum`  & voorspeltabel_maxima_kl$`Ondergrens voorspelling maximum Kliniek` < voorspeltabel_maxima_kl$`daadwerkelijk maximum`  
  
  kl_eval = tibble(
    type = paste0('Max. ',horizon,'d. ah.'),
    bias = mean(errors_kl, na.rm = T),
    MAE = mean(abs(errors_kl), na.rm = T),
    CR = mean(indc_covg_kl, na.rm = T))
  
  
  kl_eval_long = pivot_longer(kl_eval , -type)
  ic_eval_long = pivot_longer(ic_eval , -type)
  tot_eval = full_join(kl_eval_long, ic_eval_long) %>% add_column(dept = rep(c('Ward','ICU'), each = nrow(kl_eval_long)), .before = 1)  %>% mutate(value = round(value,2))
  
  
  
  
  if(strcmp(taal, 'EN')){
    
    voorspeltabel_maxima_kl = voorspeltabel_maxima_kl %>% rename(`predicted maximum occupancy at ward` = `voorspelling maximum Kliniek`, `realised maximum occupancy at ward` = `daadwerkelijk maximum`)
    plot_voorspellingen_kliniek = ggplot(voorspeltabel_maxima_kl, aes(x = datum)) +
      ylim(0, NA) +
      xlab("Date") +
      ylab(paste0("highest occupancy at ward over the last ",horizon, ' days')) +
      geom_line(aes(y = `predicted maximum occupancy at ward`), colour =colorshorizons[toString(horizon)],size = 1.2)+
      geom_line(aes(y = `realised maximum occupancy at ward`), colour = 'red',size = 1.2)+
      geom_ribbon(aes(ymax = `Bovengrens voorspelling maximum Kliniek`, ymin = `Ondergrens voorspelling maximum Kliniek`), alpha =0.2, fill = colorshorizons[toString(horizon)])+
      theme_bw()+
      scale_color_manual(values = colors)+scale_y_continuous(breaks = integer_breaks())+ylim(c(0, max(voorspeltabel_maxima_kl %>% select(-datum))+2))#+theme(legend.title = element_blank())
    
    print(ggplotly(plot_voorspellingen_kliniek))
    # schrijf_rds(plot_voorspellingen_kliniek, ziekenhuis)
    
    voorspeltabel_maxima_ic = voorspeltabel_maxima_ic %>% rename(`predicted maximum occupancy at ICU` = `voorspelling maximum IC`, `realised maximum occupancy at ICU` = `daadwerkelijk maximum`)
    
    plot_voorspellingen_ic = ggplot(voorspeltabel_maxima_ic, aes(x = datum)) +
      ylim(0, NA) +
      xlab("Date") +
      ylab(paste0("highest occupancy at ICU over the last ",horizon, ' days')) +
      geom_line(aes(y = `predicted maximum occupancy at ICU`),size = 1.2, colour = colorshorizons[toString(horizon)])+
      geom_line(aes(y = `realised maximum occupancy at ICU`),size = 1.2, colour = 'red')+
      geom_ribbon(aes(ymax = `Bovengrens voorspelling maximum IC`, ymin = `Ondergrens voorspelling maximum IC`), alpha =0.2, fill = colorshorizons[toString(horizon)])+
      theme_bw()+scale_y_continuous(breaks = integer_breaks())+ylim(c(0,max(voorspeltabel_maxima_ic %>% select(-datum))+2))#+theme(legend.title = element_blank())
    
    print(ggplotly(plot_voorspellingen_ic))
  }else{
    plot_voorspellingen_kliniek = ggplot(voorspeltabel_maxima_kl, aes(x = datum)) +
      ylim(0, NA) +
      xlab("Datum") +
      ylab(paste0("maximale bedbenutting Kliniek")) +
      geom_line(aes(y = `voorspelling maximum Kliniek`, colour = 'voorspelling maximum Kliniek'),size = 1.2)+
      geom_line(aes(y = `daadwerkelijk maximum`, colour = 'daadwerkelijk maximum Kliniek'),size = 1.2)+
      geom_ribbon(aes(ymax = `Bovengrens voorspelling maximum Kliniek`, ymin = `Ondergrens voorspelling maximum Kliniek`), alpha =0.2, fill = 'blue')+
      theme_bw()+ylim(c(0, max(voorspeltabel_maxima_kl$`daadwerkelijk maximum`)+2))#+theme(legend.title = element_blank())
    
    print(ggplotly(plot_voorspellingen_kliniek))
    # schrijf_rds(plot_voorspellingen_kliniek, ziekenhuis)
    
    plot_voorspellingen_ic = ggplot(voorspeltabel_maxima_ic, aes(x = datum)) +
      ylim(0, NA) +
      xlab("Datum") +
      ylab(paste0("maximale bedbenutting IC in de laatste ", horizon, ' dagen')) +
      geom_line(aes(y = `voorspelling maximum IC`), colour = colors['voorspelling maximum IC'],size = 1.2)+
      geom_line(aes(y = `daadwerkelijk maximum`), colour = colors['daadwerkelijk maximum IC'],size = 1.2)+
      geom_ribbon(aes(ymax = `Bovengrens voorspelling maximum IC`, ymin = `Ondergrens voorspelling maximum IC`), alpha =0.2, fill = 'red')+
      theme_bw()+theme(legend.title = element_blank())+ylim(c(0, max(voorspeltabel_maxima_ic$`daadwerkelijk maximum`)+2))
    
    print(ggplotly(plot_voorspellingen_ic))
  }
  
  if(save_plots){
    ggsave(paste0('R/data analyse/data/maxima_plot_ic_', ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),horizon, '.pdf'),plot_voorspellingen_ic, device = cairo_pdf, width = 125, height = 125, units = 'mm')
    ggsave(paste0('R/data analyse/data/maxima_plot_kl_', ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),horizon, '.pdf'),plot_voorspellingen_kliniek, device = cairo_pdf, width = 125, height = 125, units = 'mm')
  }
  
  return(tot_eval)
}


maak_plots_maxima_oud = function(summaries, Kliniekset, ICset, horizon, overplaatsingen_er_uit,  taal  ='EN', save_plots = T, CI_val = 0.95, max_date = as.Date('2020-07-10')){
  preds = c()
  dates = c()
  realisatie_ic = c()
  realisatie_kl = c()
  for(datestring in names(summaries)){
    if(as.Date(datestring)<= max_date){
    dates = c(dates, toString(as.Date(datestring) + days(horizon)))
    maximum_ic = apply(summaries[[datestring]]$maxima_IC, 2, mean)
    ub_maximum_ic = apply(summaries[[datestring]]$maxima_IC, 2, function(v){quantile(v, 1-(1-CI_val)/2)})
    lb_maximum_ic = apply(summaries[[datestring]]$maxima_IC, 2, function(v){quantile(v, (1-CI_val)/2)})
    maxima_kl = apply(summaries[[datestring]]$maxima_kl, 2, mean)
    ub_maxima_kl = apply(summaries[[datestring]]$maxima_kl, 2, function(v){quantile(v, 1-(1-CI_val)/2)})
    lb_maxima_kl = apply(summaries[[datestring]]$maxima_kl, 2, function(v){quantile(v, (1-CI_val)/2)})
    preds = rbind(preds, c(maximum_ic, maxima_kl, ub_maximum_ic, ub_maxima_kl, lb_maximum_ic, lb_maxima_kl))
    realisatie_ic = c(realisatie_ic, summaries[[datestring]]$pred_df_ic %>% filter(datum == datestring) %>% pull('avg_log'))
    realisatie_kl = c(realisatie_kl, summaries[[datestring]]$pred_df_kl %>% filter(datum == datestring) %>% pull('avg_log'))
  }}
  pred_tbl = tibble(datum = as.Date(dates), `voorspelling maximum IC` = preds[,1],  `voorspelling maximum Kliniek` = preds[,2],
                    `Bovengrens voorspelling maximum IC` = preds[,3], `Bovengrens voorspelling maximum Kliniek` = preds[,4],
                    `Ondergrens voorspelling maximum IC` = preds[,5], `Ondergrens voorspelling maximum Kliniek` = preds[,6],)
  pred_tbl = pred_tbl %>% filter(datum <= max_date)

  historische_maximale_bezetting  = list(hist_bezetting_ic = tibble(datum = as.Date(names(summaries)[as.Date(names(summaries))<= max_date]), bezetting = realisatie_ic),
                                         hist_bezetting_kl = tibble(datum = as.Date(names(summaries)[as.Date(names(summaries))<= max_date]), bezetting = realisatie_kl))
  maxfn = rollify(max, window = horizon)
  historische_maximale_bezetting$hist_bezetting_ic = historische_maximale_bezetting$hist_bezetting_ic %>% mutate(bezetting = maxfn(bezetting))
  historische_maximale_bezetting$hist_bezetting_kl = historische_maximale_bezetting$hist_bezetting_kl %>% mutate(bezetting = maxfn(bezetting))
  
  daadwerkelijke_maxima_IC = historische_maximale_bezetting$hist_bezetting_ic%>% filter(as.Date(datum) %in% pred_tbl$datum)
  daadwerkelijke_maxima_Kliniek = historische_maximale_bezetting$hist_bezetting_kl %>% filter(as.Date(datum) %in% pred_tbl$datum)
  
  voorspeltabel_maxima_ic = pred_tbl[-seq(horizon),] %>% select(-`voorspelling maximum Kliniek`,
                                                -`Bovengrens voorspelling maximum Kliniek`,
                                                -`Ondergrens voorspelling maximum Kliniek`) %>% left_join( daadwerkelijke_maxima_IC, by = 'datum')%>%
                                    rename(`daadwerkelijk maximum` = bezetting)
  
  voorspeltabel_maxima_kl = pred_tbl[-seq(horizon),] %>% select(-`voorspelling maximum IC`,
                                                -`Bovengrens voorspelling maximum IC`,
                                                -`Ondergrens voorspelling maximum IC`)  %>% left_join( daadwerkelijke_maxima_Kliniek, by = 'datum') %>% 
                                      rename(`daadwerkelijk maximum` = bezetting)
  
  
  #haal fout/coverage resultaten op voor IC
  errors_ic = voorspeltabel_maxima_ic$`voorspelling maximum IC` - voorspeltabel_maxima_ic$`daadwerkelijk maximum`
  indc_covg_ic = voorspeltabel_maxima_ic$`Bovengrens voorspelling maximum IC` >= voorspeltabel_maxima_ic$`daadwerkelijk maximum` &voorspeltabel_maxima_ic$`Ondergrens voorspelling maximum IC` <= voorspeltabel_maxima_ic$`daadwerkelijk maximum`  
                  
  ic_eval = tibble(
    type = paste0('Max. ',horizon,'d. ah.'),
    bias = mean(errors_ic, na.rm = T),
    MAE = mean(abs(errors_ic), na.rm=T),
    CR = mean(indc_covg_ic, na.rm=T))
  
  #haal fout/coverage resultaten op voor Kliniek
  errors_kl = voorspeltabel_maxima_kl$`voorspelling maximum Kliniek` - voorspeltabel_maxima_kl$`daadwerkelijk maximum`
  indc_covg_kl = voorspeltabel_maxima_kl$`Bovengrens voorspelling maximum Kliniek` > voorspeltabel_maxima_kl$`daadwerkelijk maximum`  & voorspeltabel_maxima_kl$`Ondergrens voorspelling maximum Kliniek` < voorspeltabel_maxima_kl$`daadwerkelijk maximum`  
                 
  kl_eval = tibble(
    type = paste0('Max. ',horizon,'d. ah.'),
    bias = mean(errors_kl, na.rm = T),
    MAE = mean(abs(errors_kl), na.rm = T),
    CR = mean(indc_covg_kl, na.rm = T))
  
  
  kl_eval_long = pivot_longer(kl_eval , -type)
  ic_eval_long = pivot_longer(ic_eval , -type)
  tot_eval = full_join(kl_eval_long, ic_eval_long) %>% add_column(dept = rep(c('Ward','ICU'), each = nrow(kl_eval_long)), .before = 1)  %>% mutate(value = round(value,2))
  
  
  

    if(strcmp(taal, 'EN')){
    
    voorspeltabel_maxima_kl = voorspeltabel_maxima_kl %>% rename(`predicted maximum occupancy at ward` = `voorspelling maximum Kliniek`, `realised maximum occupancy at ward` = `daadwerkelijk maximum`)
    plot_voorspellingen_kliniek = ggplot(voorspeltabel_maxima_kl, aes(x = datum)) +
                                             ylim(0, NA) +
                                             xlab("Date") +
                                             ylab(paste0("highest occupancy at ward over the last ",horizon, ' days')) +
                                             geom_line(aes(y = `predicted maximum occupancy at ward`), colour =colorshorizons[toString(horizon)],size = 1.2)+
                                             geom_line(aes(y = `realised maximum occupancy at ward`), colour = 'red',size = 1.2)+
                                             geom_ribbon(aes(ymax = `Bovengrens voorspelling maximum Kliniek`, ymin = `Ondergrens voorspelling maximum Kliniek`), alpha =0.2, fill = colorshorizons[toString(horizon)])+
                                             theme_bw()+theme(legend.title = element_blank())+scale_y_continuous(breaks = integer_breaks())+ylim(c(0, max(voorspeltabel_maxima_kl %>% select(-datum))+2))
    
    print(ggplotly(plot_voorspellingen_kliniek))
    # schrijf_rds(plot_voorspellingen_kliniek, ziekenhuis)
    
    voorspeltabel_maxima_ic = voorspeltabel_maxima_ic %>% rename(`predicted maximum occupancy at ICU` = `voorspelling maximum IC`, `realised maximum occupancy at ICU` = `daadwerkelijk maximum`)
    
    plot_voorspellingen_ic = ggplot(voorspeltabel_maxima_ic, aes(x = datum)) +
                                        ylim(0, NA) +
                                        xlab("Date") +
                                        ylab(paste0("highest occupancy at ICU over the last ",horizon, ' days')) +
                                        geom_line(aes(y = `predicted maximum occupancy at ICU`),size = 1.2, colour = colorshorizons[toString(horizon)])+
                                        geom_line(aes(y = `realised maximum occupancy at ICU`),size = 1.2, colour = 'red')+
                                        geom_ribbon(aes(ymax = `Bovengrens voorspelling maximum IC`, ymin = `Ondergrens voorspelling maximum IC`), alpha =0.2, fill = colorshorizons[toString(horizon)])+
                                        theme_bw()+theme(legend.title = element_blank())+scale_y_continuous(breaks = integer_breaks())+ylim(c(0,max(voorspeltabel_maxima_ic %>% select(-datum))+2))
    
    print(ggplotly(plot_voorspellingen_ic))
  }else{
  plot_voorspellingen_kliniek = ggplot(voorspeltabel_maxima_kl, aes(x = datum)) +
    ylim(0, NA) +
    xlab("Datum") +
    ylab(paste0("maximale bedbenutting Kliniek")) +
    geom_line(aes(y = `voorspelling maximum Kliniek`, colour = 'voorspelling maximum Kliniek'),size = 1.2)+
    geom_line(aes(y = `daadwerkelijk maximum`, colour = 'daadwerkelijk maximum Kliniek'),size = 1.2)+
    geom_ribbon(aes(ymax = `Bovengrens voorspelling maximum Kliniek`, ymin = `Ondergrens voorspelling maximum Kliniek`), alpha =0.2, fill = 'blue')+
    theme_bw()+theme(legend.title = element_blank())+ylim(c(0, max(voorspeltabel_maxima_kl$`daadwerkelijk maximum`)+2))
  
  print(ggplotly(plot_voorspellingen_kliniek))
  # schrijf_rds(plot_voorspellingen_kliniek, ziekenhuis)
  
  plot_voorspellingen_ic = ggplot(voorspeltabel_maxima_ic, aes(x = datum)) +
                                           ylim(0, NA) +
                                           xlab("Datum") +
                                           ylab(paste0("maximale bedbenutting IC in de laatste ", horizon, ' dagen')) +
                                           geom_line(aes(y = `voorspelling maximum IC`), colour = colors['voorspelling maximum IC'],size = 1.2)+
                                           geom_line(aes(y = `daadwerkelijk maximum`), colour = colors['daadwerkelijk maximum IC'],size = 1.2)+
                                      geom_ribbon(aes(ymax = `Bovengrens voorspelling maximum IC`, ymin = `Ondergrens voorspelling maximum IC`), alpha =0.2, fill = 'red')+
                                      theme_bw()+theme(legend.title = element_blank())+ylim(c(0, max(voorspeltabel_maxima_ic$`daadwerkelijk maximum`)+2))

  print(ggplotly(plot_voorspellingen_ic))
  }
  
  if(save_plots){
    ggsave(paste0('R/data analyse/data/maxima_plot_ic_', ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),horizon, '.pdf'),plot_voorspellingen_ic, device = cairo_pdf, width = 125, height = 125, units = 'mm')
    ggsave(paste0('R/data analyse/data/maxima_plot_kl_', ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),horizon, '.pdf'),plot_voorspellingen_kliniek, device = cairo_pdf, width = 125, height = 125, units = 'mm')
  }
  
  return(tot_eval)
}

maak_plots_voorspellingen_pred = function(summaries, Kliniekset, ICset, scenarios, horizons = 7, save_plots = T){
  
  for(i in seq(length(horizons))){
    horizon = horizons[i]
    preds_ic = c()
    preds_kl = c()
    dates = c()
    for(datestring in names(summaries)){
      df_ic = summaries[[datestring]]$preds_bezet_tot_ic %>% select(datum, scenarios)
      row_ic = as.numeric(df_ic[df_ic$datum == as.Date(datestring) + days(horizon), ] %>% select(-datum))
      preds_ic = rbind(preds_ic, row_ic)
      
      df_kl = summaries[[datestring]]$preds_bezet_tot_kl %>% select(datum, scenarios)
      row_kl = as.numeric(df_kl[df_kl$datum == as.Date(datestring) + days(horizon), ] %>% select(-datum))
      preds_kl = rbind(preds_kl, row_kl)
      
      dates = c(dates, toString(as.Date(datestring) + days(horizon)))
    }
    
    colnames(preds_ic) = sapply(colnames(df_ic %>% select(-datum)), function(str){paste('Voorspelde Benutting IC (', horizon, ifelse(horizon>1,' dagen vooruit)', ' dag vooruit)'))})
    colnames(preds_kl) = sapply(colnames(df_kl %>% select(-datum)), function(str){paste('Voorspelde Benutting Kliniek (', horizon, ifelse(horizon>1,' dagen vooruit)', ' dag vooruit)'))})
          
              
    df_ic = as_tibble(preds_ic) %>% add_column(datum = as.Date(dates), .before = 1)
    df_kl = as_tibble(preds_kl) %>% add_column(datum = as.Date(dates), .before = 1)
    
    if(i ==1){
      pred_tbl_ic = df_ic
      pred_tbl_kl = df_kl
    }else{
      pred_tbl_ic = pred_tbl_ic %>% left_join(df_ic, by = 'datum')
      pred_tbl_kl = pred_tbl_kl %>% left_join(df_kl, by = 'datum')
    }
  }
  
  #laad functie om systeeminfo op te halen voor DES, evalueer
  source('R/get_system_information.R', encoding = "UTF-8")
  list2env(bereken_ligduren(Kliniekset = Kliniekset, ICset = ICset), envir= .GlobalEnv)
  historische_bezetting  = bereken_historische_bezetting(Kliniekset, ICset, ligduren_ic, ligduren_kl)
  # ma_pred = rollify(mean, window = horizon)
  # voorspelling_ma_ic = historische_bezetting$hist_bezetting_ic %>% mutate(bezetting = ma_pred(bezetting), datum = datum + days(horizon)) %>% filter(as.Date(datum) %in% pred_tbl_ic$datum)
  # voorspelling_ma_kl = historische_bezetting$hist_bezetting_kl %>% mutate(bezetting = ma_pred(bezetting), datum = datum + days(horizon)) %>% filter(as.Date(datum) %in% pred_tbl_kl$datum)
  
  
  daadwerkelijke_bezetting_IC = historische_bezetting$hist_bezetting_ic %>% filter(as.Date(datum) %in% pred_tbl_ic$datum)
  daadwerkelijke_bezetting_Kliniek = historische_bezetting$hist_bezetting_kl %>% filter(as.Date(datum) %in% pred_tbl_kl$datum)
  
  voorspeltabel_benutting_ic = pred_tbl_ic %>%  add_column(`daadwerkelijke benutting` = daadwerkelijke_bezetting_IC$bezetting)
  
  voorspeltabel_benutting_kl = pred_tbl_kl %>% add_column(`daadwerkelijke benutting` = daadwerkelijke_bezetting_Kliniek$bezetting)
  
  #plot data in voorspeltabel kliniek
  df_plot_kl <- melt(voorspeltabel_benutting_kl ,  id.vars = 'datum')
  plot_voorspellingen_kliniek = ggplot(df_plot_kl, aes(datum,value)) + geom_line(aes(colour = variable), size = 1.2)+
                                           ylab('BedBenutting Kliniek')+
                                            theme(legend.title = element_blank())
  print(ggplotly(plot_voorspellingen_kliniek))
  
  #plot data in voorspeltabel IC
  df_plot_ic <- melt(voorspeltabel_benutting_ic ,  id.vars = 'datum')
  plot_voorspellingen_ic = ggplot(df_plot_ic, aes(datum,value)) + geom_line(aes(colour = variable), size = 1.2)+
                                           ylab('BedBenutting IC') +
                                           theme(legend.title = element_blank())
  print(ggplotly(plot_voorspellingen_ic))
  
  if(save_plots){
    ggsave(paste0('R/data analyse/data/EWplot_voorspellingen_ic_', ziekenhuis, '.pdf'),plot_voorspellingen_ic, device = cairo_pdf, width = 300, height = 150, units = 'mm')
    ggsave(paste0('R/data analyse/data/EWplot_voorspellingen_kl_', ziekenhuis, '.pdf'),plot_voorspellingen_kliniek, device = cairo_pdf, width = 300, height = 150, units = 'mm')
  }
  return(mget(c('voorspeltabel_benutting_kl', 'voorspeltabel_benutting_ic')))
}


maak_plots_voorspellingen_sim_EN = function(summaries, Kliniekset, ICset, scenarios, horizons ,overplaatsingen_er_uit, save_plots = T, CI_plot = T,
                                            max_date = as.Date('2020-07-10')){
  #haal voorspellingen op + boven/ondergrens
  
  horizons = c(0, horizons) #voeg horizon 0 toe om huidige bezetting uit te rekenen
  for(i in seq(length(horizons))){
    horizon = horizons[i]
    preds_ic = c()
    lb_ic = c()
    ub_ic = c()
    preds_kl = c()
    lb_kl = c()
    ub_kl = c()
    dates = c()
    for(datestring in names(summaries)){
      if(as.Date(datestring) <= max_date){
      df_ic = summaries[[datestring]]$pred_df_ic %>% select(datum, sapply(scenarios, function(str){paste0('avg_', str)}))
      row_ic = as.numeric(df_ic[df_ic$datum == as.Date(datestring) + days(horizon), ] %>% select(-datum))
      preds_ic = rbind(preds_ic, row_ic)
      
      df_lb_ic = summaries[[datestring]]$pred_df_ic %>% select(datum, sapply(scenarios, function(str){paste0('lbCI95%_', str)}))
      row_lb_ic = as.numeric(df_lb_ic[df_lb_ic$datum == as.Date(datestring) + days(horizon), ] %>% select(-datum))
      lb_ic = rbind(lb_ic, row_lb_ic)
      
      df_ub_ic = summaries[[datestring]]$pred_df_ic %>% select(datum, sapply(scenarios, function(str){paste0('ubCI95%_', str)}))
      row_ub_ic = as.numeric(df_ub_ic[df_ub_ic$datum == as.Date(datestring) + days(horizon), ] %>% select(-datum))
      ub_ic = rbind(ub_ic, row_ub_ic)
      
      df_kl = summaries[[datestring]]$pred_df_kl %>% select(datum, sapply(scenarios, function(str){paste0('avg_', str)}))
      row_kl = as.numeric(df_kl[df_kl$datum == as.Date(datestring) + days(horizon), ] %>% select(-datum))
      preds_kl = rbind(preds_kl, row_kl)
      
      df_lb_kl = summaries[[datestring]]$pred_df_kl %>% select(datum, sapply(scenarios, function(str){paste0('lbCI95%_', str)}))
      row_lb_kl = as.numeric(df_lb_kl[df_lb_kl$datum == as.Date(datestring) + days(horizon), ] %>% select(-datum))
      lb_kl = rbind(lb_kl, row_lb_kl)
      
      df_ub_kl = summaries[[datestring]]$pred_df_kl %>% select(datum, sapply(scenarios, function(str){paste0('ubCI95%_', str)}))
      row_ub_kl = as.numeric(df_ub_kl[df_ub_kl$datum == as.Date(datestring) + days(horizon), ] %>% select(-datum))
      ub_kl = rbind(ub_kl, row_ub_kl)
      
      dates = c(dates, toString(as.Date(datestring) + days(horizon)))}
    }
    
    colnames(preds_ic) = sapply(colnames(df_ic %>% select(-datum)), function(str){paste('Predicted occupancy ICU (', horizon, ifelse(horizon>1,' days ahead)', ' day ahead)'))})
    colnames(preds_kl) = sapply(colnames(df_kl %>% select(-datum)), function(str){paste('Predicted occupancy Ward (', horizon, ifelse(horizon>1,' days ahead)', ' day ahead)'))})
    
    colnames(lb_ic) = sapply(colnames(df_ic %>% select(-datum)), function(str){paste('Predicted occupancy ICU (', horizon, ifelse(horizon>1,' days ahead)', ' day ahead)'))})
    colnames(lb_kl) = sapply(colnames(df_kl %>% select(-datum)), function(str){paste('Predicted occupancy Ward (', horizon, ifelse(horizon>1,' days ahead)', ' day ahead)'))})
    
    colnames(ub_ic) = sapply(colnames(df_ic %>% select(-datum)), function(str){paste('Predicted occupancy ICU (', horizon, ifelse(horizon>1,' days ahead)', ' day ahead)'))})
    colnames(ub_kl) = sapply(colnames(df_kl %>% select(-datum)), function(str){paste('Predicted occupancy Ward (', horizon, ifelse(horizon>1,' days ahead)', ' day ahead)'))})
    
    
    df_ic = as_tibble(preds_ic) %>% add_column(datum = as.Date(dates), .before = 1)
    ub_ic = as_tibble(ub_ic) %>% add_column(datum = as.Date(dates), .before = 1)
    lb_ic = as_tibble(lb_ic) %>% add_column(datum = as.Date(dates), .before = 1)
    df_kl = as_tibble(preds_kl) %>% add_column(datum = as.Date(dates), .before = 1)
    ub_kl = as_tibble(ub_kl) %>% add_column(datum = as.Date(dates), .before = 1)
    lb_kl = as_tibble(lb_kl) %>% add_column(datum = as.Date(dates), .before = 1)
    
    if(i ==1){
      pred_tbl_ic = df_ic
      ub_tbl_ic = ub_ic
      lb_tbl_ic = lb_ic
      pred_tbl_kl = df_kl
      ub_tbl_kl = ub_kl
      lb_tbl_kl = lb_kl
    }else{
      pred_tbl_ic = pred_tbl_ic %>% left_join(df_ic, by = 'datum') 
      lb_tbl_ic = lb_tbl_ic %>% left_join(lb_ic, by = 'datum')
      ub_tbl_ic = ub_tbl_ic %>% left_join(ub_ic, by = 'datum')
      pred_tbl_kl = pred_tbl_kl %>% left_join(df_kl, by = 'datum')
      lb_tbl_kl = lb_tbl_kl %>% left_join(lb_kl, by = 'datum')
      ub_tbl_kl = ub_tbl_kl %>% left_join(ub_kl, by = 'datum')
    }
  }
  horizons = horizons[-1] # horizon 0 weer weghalen

  voorspeltabel_benutting_ic = pred_tbl_ic %>%  rename(`Realised occupancy at ICU` = `Predicted occupancy ICU ( 0  day ahead)`)%>% rename(date = datum)
  voorspeltabel_benutting_kl = pred_tbl_kl %>% rename(`Realised occupancy at Ward` = `Predicted occupancy Ward ( 0  day ahead)`)%>% rename(date = datum)
  
  pred_tbl_ic = pred_tbl_ic %>% rename(date = datum)
  lb_tbl_ic = lb_tbl_ic %>% rename(date = datum)
  ub_tbl_ic = ub_tbl_ic %>% rename(date = datum)
  pred_tbl_kl = pred_tbl_kl %>% rename(date = datum)
  lb_tbl_kl = lb_tbl_kl %>% rename(date = datum)
  ub_tbl_kl = ub_tbl_kl%>% rename(date = datum)
  
  #plot data in voorspeltabel kliniek
  df_pred_plot_kl <- melt(voorspeltabel_benutting_kl ,  id.vars = 'date')
  df_ub_plot_kl <- melt(ub_tbl_kl ,  id.vars = 'date') %>% rename( Bovengrens = value)
  df_lb_plot_kl <- melt(lb_tbl_kl ,  id.vars = 'date') %>% rename(Ondergrens = value)
  df_plot_kl = df_pred_plot_kl %>% left_join(df_ub_plot_kl, by = c('date', 'variable')) %>% left_join(df_lb_plot_kl, by = c('date', 'variable'))
  
  
  #plot data in voorspeltabel kliniek
  df_pred_plot_ic <- melt(voorspeltabel_benutting_ic ,  id.vars = 'date')
  df_ub_plot_ic <- melt(ub_tbl_ic ,  id.vars = 'date') %>% rename( Bovengrens = value)
  df_lb_plot_ic <- melt(lb_tbl_ic ,  id.vars = 'date') %>% rename(Ondergrens = value)
  df_plot_ic = df_pred_plot_ic %>% left_join(df_ub_plot_ic, by = c('date', 'variable')) %>%
    left_join(df_lb_plot_ic, by = c('date', 'variable'))
  
  colorshorizons_kliniek = colorshorizons
  names(colorshorizons_kliniek) = sapply(names(colorshorizons), function(hor){paste0('Predicted occupancy Ward ( ',hor,ifelse(strcmp(hor,'1'), '  day ahead)','  days ahead)'))})
  plot_voorspellingen_kliniek = ggplot(df_plot_kl, aes(date,value)) + geom_line(aes(colour = variable), size = 1.2)+
    ylab('occupancy at Ward')+ theme_bw()+
    theme(legend.title=element_blank())+#legend.position = 'none'
    scale_colour_manual(values = c('Realised occupancy at Ward' = 'red', colorshorizons_kliniek))+
    scale_y_continuous(breaks = integer_breaks())+ylim(c(0, max(df_plot_kl$value)+2))
  
  
  colorshorizons_ic = colorshorizons
  names(colorshorizons_ic) = sapply(names(colorshorizons), function(hor){paste0('Predicted occupancy ICU ( ',hor,ifelse(strcmp(hor,'1'), '  day ahead)','  days ahead)'))})
  plot_voorspellingen_ic = ggplot(df_plot_ic, aes(date,value)) + geom_line(aes(colour = variable), size = 1.2)+
    ylab('occupancy at ICU')+ theme_bw()+
     theme(legend.title=element_blank())+#legend.position = 'none'
    scale_colour_manual(values = c('Realised occupancy at ICU' = 'red', colorshorizons_ic))+
    scale_y_continuous(breaks = integer_breaks())+ylim(c(0, max(df_plot_ic$value)+2))
  
  if(CI_plot){
    plot_voorspellingen_ic = plot_voorspellingen_ic +geom_ribbon(aes(ymin = Ondergrens, ymax =Bovengrens), alpha = 0.2, na.rm = T, fill = 'blue') 
    plot_voorspellingen_kliniek = plot_voorspellingen_kliniek +geom_ribbon(aes(ymin = Ondergrens, ymax =Bovengrens), alpha = 0.2, na.rm = T, fill = 'blue')
    }
  print(ggplotly(plot_voorspellingen_kliniek))
  print(ggplotly(plot_voorspellingen_ic))
  
  
  if(save_plots){
    ggsave(paste0('R/data analyse/data/EWplot_voorspellingen_ic_',ifelse(CI_plot, 'CI_',''), ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),max(horizons), '.pdf'),plot_voorspellingen_ic, device = cairo_pdf, width = 125, height = 125, units = 'mm')
    ggsave(paste0('R/data analyse/data/EWplot_voorspellingen_kl_',ifelse(CI_plot, 'CI_',''), ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),max(horizons), '.pdf'),plot_voorspellingen_kliniek, device = cairo_pdf, width = 125, height = 125, units = 'mm')
  }
  
  #haal fout/coverage resultaten op voor IC
  lb_tbl_ic = lb_tbl_ic %>% select(-`Predicted occupancy ICU ( 0  day ahead)`)
  ub_tbl_ic = ub_tbl_ic %>% select(-`Predicted occupancy ICU ( 0  day ahead)`)
  errors_ic = voorspeltabel_benutting_ic[-seq(max(horizons)),] %>% select(-date, -`Realised occupancy at ICU`) - voorspeltabel_benutting_ic[-seq(max(horizons)),rep("Realised occupancy at ICU", ncol(voorspeltabel_benutting_ic) - 2)]
  indc_covg_ic = ub_tbl_ic[-seq(max(horizons)),] %>% select(-date) >= voorspeltabel_benutting_ic[-seq(max(horizons)),rep("Realised occupancy at ICU", ncol(voorspeltabel_benutting_ic) - 2)]&lb_tbl_ic[-seq(max(horizons)),] %>% select(-date) <= voorspeltabel_benutting_ic[-seq(max(horizons)),rep("Realised occupancy at ICU", ncol(voorspeltabel_benutting_ic) - 2)]
    
  ic_eval = tibble(
    type = sapply(horizons, function(days){paste0(days, ifelse(days == 1, ' day ah.', ' days ah.'))}),
    bias = apply(errors_ic,2,function(v){mean((v), na.rm=T)}),
    MAE = apply(errors_ic,2,function(v){mean(abs(v), na.rm=T)}),
    CR = apply(indc_covg_ic, 2, mean))
  
  #haal fout/coverage resultaten op voor Kliniek
  lb_tbl_kl = lb_tbl_kl %>% select(-`Predicted occupancy Ward ( 0  day ahead)`)
  ub_tbl_kl = ub_tbl_kl %>% select(-`Predicted occupancy Ward ( 0  day ahead)`)
  errors_kl = voorspeltabel_benutting_kl[-seq(max(horizons)),] %>% select(-date, -`Realised occupancy at Ward`) - voorspeltabel_benutting_kl[-seq(max(horizons)),rep("Realised occupancy at Ward", ncol(voorspeltabel_benutting_kl) - 2)]
  indc_covg_kl = ub_tbl_kl[-seq(max(horizons)),] %>% select(-date) > voorspeltabel_benutting_kl[-seq(max(horizons)),rep("Realised occupancy at Ward", ncol(voorspeltabel_benutting_kl) - 2)]&lb_tbl_kl[-seq(max(horizons)),] %>% select(-date) < voorspeltabel_benutting_kl[-seq(max(horizons)),rep("Realised occupancy at Ward", ncol(voorspeltabel_benutting_kl) - 2)]
    
  kl_eval = tibble(
    type = sapply(horizons, function(days){paste0(days, ifelse(days == 1, ' day ah.', ' days ah.'))}),
    bias = apply(errors_kl,2,function(v){mean((v), na.rm=T)}),
    MAE = apply(errors_kl,2,function(v){mean(abs(v), na.rm=T)}),
    CR = apply(indc_covg_kl, 2, mean))
  
  #make the table long (for writing to  LaTeX)
  horizons_int = rep(horizons, each = ncol(ic_eval)-1)
  ic_eval_long = pivot_longer(ic_eval, -type) 
  kl_eval_long = pivot_longer(kl_eval, -type)
  departments   = rep(c("ICU", "Ward"), each = nrow(ic_eval_long))
  tot_eval = full_join(ic_eval_long, kl_eval_long) %>% add_column(dept = departments, .before = 1) %>% mutate(value = round(value,2))
  
  #make weekly average predictions 
  ma_pred_wk = rollify(mean, window = 7) # get weekly moving average fn
  voorspelling_ma_ic = ma_pred_wk(voorspeltabel_benutting_ic$`Realised occupancy at ICU`)
  voorspelling_ma_kl = ma_pred_wk(voorspeltabel_benutting_kl$`Realised occupancy at Ward`)
  
  
  errors_ic_ma =  sapply(horizons, function(horizon){c(rep(NA, horizon), head(voorspelling_ma_ic, length(voorspelling_ma_ic)- horizon))})[-seq(max(horizons)),] - voorspeltabel_benutting_ic[-seq(max(horizons)),rep("Realised occupancy at ICU", ncol(voorspeltabel_benutting_ic) - 2)]
  ic_eval_ma = tibble(
    type = sapply(horizons, function(days){paste0(days, ifelse(days == 1, ' day ah.', ' days ah.'))}),
    bias = apply(errors_ic_ma,2,function(v){mean((v), na.rm=T)}),
    MAE = apply(errors_ic_ma,2,function(v){mean(abs(v), na.rm=T)}))
    
  errors_kl_ma =  sapply(horizons, function(horizon){c(rep(NA, horizon), head(voorspelling_ma_kl, length(voorspelling_ma_kl)- horizon))})[-seq(max(horizons)),] - voorspeltabel_benutting_kl[-seq(max(horizons)),rep("Realised occupancy at Ward", ncol(voorspeltabel_benutting_kl) - 2)]
  kl_eval_ma = tibble(
    type = sapply(horizons, function(days){paste0(days, ifelse(days == 1, ' day ah.', ' days ah.'))}),
    bias = apply(errors_kl_ma,2,function(v){mean((v), na.rm=T)}),
    MAE = apply(errors_kl_ma,2,function(v){mean(abs(v), na.rm=T)}))
  
  #make the table long (for writing to  LaTeX)
  horizons_int = rep(horizons, each = ncol(ic_eval)-1)
  ic_eval_long_ma = pivot_longer(ic_eval_ma, -type) 
  kl_eval_long_ma = pivot_longer(kl_eval_ma, -type)
  departments   = rep(c("ICU", "Ward"), each = nrow(ic_eval_long_ma))
  tot_eval_ma = full_join(ic_eval_long_ma, kl_eval_long_ma) %>% add_column(dept = departments, .before = 1) %>% mutate(value = round(value,2))
  
  return(mget(c('tot_eval', 'tot_eval_ma', 'df_pred_plot_kl', 'df_pred_plot_ic')))
}

maak_voorspeltabel_det = function(summaries, Kliniekset, ICset, scenarios, horizons ,overplaatsingen_er_uit, save_plots = T,
                                  max_date = as.Date('2020-07-10')){
  #haal voorspellingen op + boven/ondergrens
  
  horizons = c(0, horizons) #voeg horizon 0 toe om huidige bezetting uit te rekenen
  for(i in seq(length(horizons))){
    horizon = horizons[i]
    preds_ic = c()
    preds_kl = c()
    dates = c()
    for(datestring in names(summaries)){
      if(as.Date(datestring) <= max_date){
        df_ic = summaries[[datestring]]$preds_bezet_tot_ic %>% select(datum, scenarios)
        row_ic = as.numeric(df_ic[df_ic$datum == as.Date(datestring) + days(horizon), ] %>% select(-datum))
        preds_ic = rbind(preds_ic, row_ic)
        
        df_kl = summaries[[datestring]]$preds_bezet_tot_kl %>% select(datum, scenarios)
        row_kl = as.numeric(df_kl[df_kl$datum == as.Date(datestring) + days(horizon), ] %>% select(-datum))
        preds_kl = rbind(preds_kl, row_kl)
        
        dates = c(dates, toString(as.Date(datestring) + days(horizon)))}
    }
    
    colnames(preds_ic) = sapply(colnames(df_ic %>% select(-datum)), function(str){paste('Predicted occupancy ICU (', horizon, ifelse(horizon>1,' days ahead)', ' day ahead)'))})
    colnames(preds_kl) = sapply(colnames(df_kl %>% select(-datum)), function(str){paste('Predicted occupancy Ward (', horizon, ifelse(horizon>1,' days ahead)', ' day ahead)'))})
    
    
    df_ic = as_tibble(preds_ic) %>% add_column(date = as.Date(dates), .before = 1)
    df_kl = as_tibble(preds_kl) %>% add_column(date = as.Date(dates), .before = 1)
    
    if(i ==1){
      pred_tbl_ic = df_ic
      pred_tbl_kl = df_kl
    }else{
      pred_tbl_ic = pred_tbl_ic %>% left_join(df_ic, by = 'date') 
      pred_tbl_kl = pred_tbl_kl %>% left_join(df_kl, by = 'date')
    }
  }
  horizons = horizons[-1] # horizon 0 weer weghalen
  
  voorspeltabel_benutting_ic = pred_tbl_ic %>%  rename(`Realised occupancy at ICU` = `Predicted occupancy ICU ( 0  day ahead)`)
  voorspeltabel_benutting_kl = pred_tbl_kl %>% rename(`Realised occupancy at Ward` = `Predicted occupancy Ward ( 0  day ahead)`)
  
  #haal fout/coverage resultaten op voor IC
  errors_ic = voorspeltabel_benutting_ic[-seq(max(horizons)),] %>% select(-date, -`Realised occupancy at ICU`) - voorspeltabel_benutting_ic[-seq(max(horizons)),rep("Realised occupancy at ICU", ncol(voorspeltabel_benutting_ic) - 2)]
  
  ic_eval = tibble(
    type = sapply(horizons, function(days){paste0(days, ifelse(days == 1, ' day ah.', ' days ah.'))}),
    bias = apply(errors_ic,2,function(v){mean((v), na.rm=T)}),
    MAE = apply(errors_ic,2,function(v){mean(abs(v), na.rm=T)}))
  
  #haal fout/coverage resultaten op voor Kliniek
  errors_kl = voorspeltabel_benutting_kl[-seq(max(horizons)),] %>% select(-date, -`Realised occupancy at Ward`) - voorspeltabel_benutting_kl[-seq(max(horizons)),rep("Realised occupancy at Ward", ncol(voorspeltabel_benutting_kl) - 2)]
  
  kl_eval = tibble(
    type = sapply(horizons, function(days){paste0(days, ifelse(days == 1, ' day ah.', ' days ah.'))}),
    bias = apply(errors_kl,2,function(v){mean((v), na.rm=T)}),
    MAE = apply(errors_kl,2,function(v){mean(abs(v), na.rm=T)}))
  
  #make the table long (for writing to  LaTeX)
  horizons_int = rep(horizons, each = ncol(ic_eval)-1)
  ic_eval_long = pivot_longer(ic_eval, -type) 
  kl_eval_long = pivot_longer(kl_eval, -type)
  departments   = rep(c("ICU", "Ward"), each = nrow(ic_eval_long))
  tot_eval = full_join(ic_eval_long, kl_eval_long) %>% add_column(dept = departments, .before = 1) %>% mutate(value = round(value,2))
  
  #make weekly average predictions 
  ma_pred_wk = rollify(mean, window = 7) # get weekly moving average fn
  voorspelling_ma_ic = ma_pred_wk(voorspeltabel_benutting_ic$`Realised occupancy at ICU`)
  voorspelling_ma_kl = ma_pred_wk(voorspeltabel_benutting_kl$`Realised occupancy at Ward`)
  
  
  errors_ic_ma =  rep.col(as.matrix(voorspelling_ma_ic),length(horizons)) -  sapply(horizons, function(hor){c((voorspeltabel_benutting_ic %>% pull("Realised occupancy at ICU"))[-seq(hor)], rep(NA, hor))})
  ic_eval_ma = tibble(
    type = sapply(horizons, function(days){paste0(days, ifelse(days == 1, ' day ah.', ' days ah.'))}),
    bias = apply(errors_ic_ma,2,function(v){mean((v), na.rm=T)}),
    MAE = apply(errors_ic_ma,2,function(v){mean(abs(v), na.rm=T)}))
  
  errors_kl_ma =  rep.col(as.matrix(voorspelling_ma_kl),length(horizons)) -  sapply(horizons, function(hor){c((voorspeltabel_benutting_kl %>% pull("Realised occupancy at Ward"))[-seq(hor)], rep(NA, hor))})
  kl_eval_ma = tibble(
    type = sapply(horizons, function(days){paste0(days, ifelse(days == 1, ' day ah.', ' days ah.'))}),
    bias = apply(errors_kl_ma,2,function(v){mean((v), na.rm=T)}),
    MAE = apply(errors_kl_ma,2,function(v){mean(abs(v), na.rm=T)}))
  
  #make the table long (for writing to  LaTeX)
  horizons_int = rep(horizons, each = ncol(ic_eval)-1)
  ic_eval_long_ma = pivot_longer(ic_eval_ma, -type) 
  kl_eval_long_ma = pivot_longer(kl_eval_ma, -type)
  departments   = rep(c("ICU", "Ward"), each = nrow(ic_eval_long_ma))
  tot_eval_ma = full_join(ic_eval_long_ma, kl_eval_long_ma) %>% add_column(dept = departments, .before = 1) %>% mutate(value = round(value,2))
  
  return(mget(c('tot_eval', 'tot_eval_ma')))
}

if(sys.nframe() == 0){
  # hier stel je param in voor plots, eerst moet "haal_voorspellingen_op.R" met deze parameters gedraaid worden en er moet dan een .rds file weggeschreven zijn met deze parameter instellingen
  ziekenhuis = 'ETZ'
  scenarios = c('log')
  if(ziekenhuis == 'LUMC'){
    horizons = c(1,2,3,5)
  }else{
    horizons = c(1,3)
  }
  bool_save = F
  horizon_CI_plot = 3
  app = '_adaptive'
  horizon_max = 3
  aantal_dagen = 5
  overplaatsingen_er_uit = T
  load(paste0('data/data manuscript/rolling_preds_',ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),aantal_dagen,app,'.Rds'))
  
  source('R/00_maak_datasets.R')
  list2env(maak_datasets(ziekenhuis), envir = .GlobalEnv)
  
  #maak plots en haal foutentabel op voor sim voorspeller
  res_dag = maak_plots_voorspellingen_sim_EN(summaries, Kliniekset, ICset, scenarios, horizons, overplaatsingen_er_uit, CI_plot = F, save_plots = bool_save)
  # maak_plots_voorspellingen_sim_EN(summaries, Kliniekset, ICset, scenarios,horizon_CI_plot  ,overplaatsingen_er_uit , CI_plot = T, save_plots = bool_save)
  
  # #haal foutentabel op voor det voorspeller
  # load(paste0('R/data analyse/data/rolling_preds_',ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),aantal_dagen,'_det.Rds'))
  # res_dag_det = maak_voorspeltabel_det(summaries, Kliniekset, ICset, scenarios, horizons, overplaatsingen_er_uit)
  # 
  # res_dag = res_dag_det$tot_eval %>% full_join(res_dag_sim$tot_eval %>% filter(name == 'CR'))
  
  # load(paste0('R/data analyse/data/rolling_preds_',ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),max(horizons),app,'.Rds'))
   
  res_max = maak_plots_maxima(summaries, Kliniekset, ICset, horizon_max, overplaatsingen_er_uit, taal = 'EN' , save_plots = bool_save)
  #  
  # res_forecaster = full_join(res_dag$tot_eval, res_max) %>% add_column(method = 'Forecast')
  # res_ma = res_dag$tot_eval_ma %>% full_join( res_max %>% mutate(value = NULL)) %>% add_column(method = 'Mov.av.forecaster')
  # tot_df_eval = full_join(res_forecaster, res_ma)
  # results_table <- tabular(RowFactor(type, name = 'type') ~
  #                            Heading()*Factor(method, levelnames = c('Forecast', 'Mov.av.forecaster'))*Heading()*Factor(dept,
  #                                             levelnames =  c("ICU",
  #                                                             "Ward" ))*
  #                            Heading()*Factor(name,
  #                                             levelnames = c('bias', 'CR', 'MAE' ))*Heading()*(value)*Heading()*(mean)*
  #                            DropEmpty(empty=""),
  #                          data = tot_df_eval)
  # results_table = results_table[,-c(8,11)]
  # results_table[is.na(results_table)] = '-'
  # results_table= results_table[,c(2,1,3,5,4,6,7,8,9,10)]
  # 
  # if(bool_save){write(tables::latex.tabular(results_table)[[1]], file = paste0('R/data analyse/data/foutentabel_',ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen', '_overplaatsingen'),'.tex'))}
}
