rm(list=ls())

# Kies ziekenhuis
ziekenhuis <- "ETZ"
start_tweede_golf = as.Date('2020-08-07') #start datum 2e golf (alle data daarvoor wordt NIET gebruikt voor schatten aankomstproces Richard's curve aangezien de dynamiek nu anders is)
scenarios = c('log')

naam_voluit = ifelse(ziekenhuis %in% c('etz', 'ETZ'), 'Elisabeth TweeSteden Ziekenhuis',
                     ifelse(ziekenhuis == 'rs', 'Rijnstate', 
                            ifelse(ziekenhuis == 'LUMC', 'LUMC', 
                                   ifelse(ziekenhuis == 'Haga', 'Haga','Onbekend'))))

# Laad packages en basisfuncties
source("R/utils.R") # basisfuncties + packages inladen
source("R/functies.R") # basisfuncties + packages inladen
source("R/functies_condprobs.R") #  basisfuncties + packages inladen

# laad functie om data in te lezen, evalueer op ziekenhuis
source("R/00_maak_datasets.R", encoding = "UTF-8")
list2env(maak_datasets(ziekenhuis), envir = .GlobalEnv)
laatste_datumtijd_opnamebestand <- as.POSIXct(datetime_pred, tz = 'UTC')
laatste_datum_opnamebestand <- as.Date(datetime_pred)
aantal_dagen = 5

# laad functies om aankomsten te voorspellen, evalueer op nieuwe opnames
source("R/01_voorspel_instroom.R", encoding = "UTF-8")
voorspelde_opnames = voorspel_instroom(df_nieuwe_opnames, aantal_dagen = aantal_dagen, scenarios, voorspeldatum = laatste_datum_opnamebestand, exclude_until = start_tweede_golf)

# laad functies om aankomsten te splitsen per afdeling, evalueer op voorspelde en geobserveerde opnames
source("R/02_voorspel_instroom_ic_kl.R", encoding = "UTF-8")
list2env(voorspel_instroom_ic_kl(voorspelde_opnames, df_nieuwe_opnames, ziekenhuis, voorspeldatum = laatste_datum_opnamebestand), envir = .GlobalEnv)

# laad functies survival analyse op ligduren, evalueer op ICset en Kliniekset
source("R/03_survival_analyse.R", encoding = "UTF-8")
list2env( survival_analyse(ziekenhuis, ICset, Kliniekset), envir= .GlobalEnv)

# laad functies voorspellen uitstroom huidige patiënten en evalueer
source("R/04_voorspel_bezetting_huidig.R", encoding = "UTF-8")
list2env(voorspel_bezetting_huidig(ziekenhuis, ICset, Kliniekset, condprobs_kl , condprobs_ic , laatste_datum_opnamebestand, aantal_dagen), envir = .GlobalEnv)

#laad functie om systeeminfo op te halen voor DES, evalueer
source('R/get_system_information.R', encoding = "UTF-8")
list2env(get_system_information(Kliniekset, ICset, preds_opnames_ic, preds_opnames_kl, 
                                condprobs_ic, condprobs_kl, condprobs_ic_kl, condprobs_kl_ic, condprobs_kl_ic_gbhbp, laatste_datumtijd_opnamebestand ), envir = .GlobalEnv)

# draai DES
source('R/DES.R')
source('R/perform_simulation_study.R', encoding = "UTF-8")
simresults = perform_simulation_study(laatste_datum_opnamebestand, aantal_dagen, list_scenario_arrival_rates, 
                                      departure_info, current_patient_info, patienttype_info, scenarios)

# vat de simulatiestudie samen in een voorspelling van de gemiddelde en maximale benutting 
source('R/summarise_simulations.R', encoding = "UTF-8")
list2env(summarise_simulations(simresults, scenarios, aantal_dagen, voorspeldatum  = laatste_datum_opnamebestand), envir = .GlobalEnv)

# maak plotjes om te gebruiken in html file
source('R/make_plots.R', encoding = "UTF-8")
make_plots(ziekenhuis, pred_df_ic, pred_df_kl , hist_bezetting_ic, hist_bezetting_kl, maxima_IC, maxima_kl,
           voorspelde_opnames, df_nieuwe_opnames, preds_opnames_ic, preds_opnames_kl,
           condprobs_ic_kl, condprobs_kl_ic, fit_kl, fit_ic, naam_voluit, ICset, Kliniekset,
           pat_per_ld_ic, pat_per_ld_kl, verwachte_bezetting_huidig_ic, verwachte_bezetting_huidig_kl, scenarios,laatste_datum_opnamebestand)

# source('R/voorspel_besmettingen.R', encoding = "UTF-8")
# try(voorspel_besmettingen_en_sla_op())


# RMarkdown ----------------------------------------------------------------------
input <- "rmarkdown/Overzicht voorspellingen_DES.Rmd"
output <- paste0("rmarkdown/", ziekenhuis, "_", laatste_datum_opnamebestand, ".html")

rmarkdown::render(
  input,
  output_file = output,
  params = list(ziekenhuis = ziekenhuis, naam_voluit = naam_voluit),
  encoding = "UTF-8"
)
