rm(list=ls())
# Laad packages en basisfuncties
source("R/utils.R") # basisfuncties + packages inladen
source("R/functies.R") # basisfuncties + packages inladen
source("R/functies_condprobs.R") #  basisfuncties + packages inladen

date = as.Date('2020-05-04')
scenarios = c('log')
ziekenhuis <- "LUMC"
aantal_dagen <- 5 #maximale horizon
overplaatsingen_er_uit = T


# laad data in voor ziekenhuis
source('R/00_maak_datasets.R')
list2env(maak_datasets(ziekenhuis), envir = .GlobalEnv)
Kliniekset$BehandelBeperking = 'Geen behandelbeperking'
ICset$BehandelBeperking = 'Geen behandelbeperking'

df_nieuwe_opnames   = df_nieuwe_opnames[df_nieuwe_opnames$datum <= date,]
Kliniekset          = maak_partiele_set(Kliniekset, date, name_dept = 'Kliniek', name_other_dept = 'IC')
ICset              = maak_partiele_set(ICset, date, name_dept = 'IC', name_other_dept = 'Kliniek')


laatste_datumtijd_opnamebestand <- max(df_nieuwe_opnames$datum)
laatste_datum_opnamebestand <- max(df_nieuwe_opnames$datum)

# laad functies om aankomsten te voorspellen, evalueer op nieuwe opnames
source("R/01_voorspel_instroom.R", encoding = "UTF-8")
voorspelde_opnames = voorspel_instroom(df_nieuwe_opnames, aantal_dagen = aantal_dagen, scenarios, voorspeldatum = laatste_datum_opnamebestand)

# laad functies om aankomsten te splitsen per afdeling, evalueer op voorspelde en geobserveerde opnames
source("R/02_voorspel_instroom_ic_kl.R", encoding = "UTF-8")
list2env(voorspel_instroom_ic_kl(voorspelde_opnames, df_nieuwe_opnames, ziekenhuis, voorspeldatum = laatste_datum_opnamebestand), envir = .GlobalEnv)

# laad functies survival analyse op ligduren, evalueer op ICset en Kliniekset
source("R/03_survival_analyse.R", encoding = "UTF-8")
list2env( survival_analyse(ziekenhuis, ICset, Kliniekset), envir= .GlobalEnv)

# laad functies voorspellen uitstroom huidige patiënten en evalueer
source("R/04_voorspel_bezetting_huidig.R", encoding = "UTF-8")
list2env(voorspel_bezetting_huidig(ziekenhuis, ICset, Kliniekset, condprobs_kl , condprobs_ic , laatste_datum_opnamebestand, aantal_dagen, overplaatsingen_er_uit = overplaatsingen_er_uit), envir = .GlobalEnv)

# Voorspel doorstroom van kliniek naar IC voor huidige patiënten
source("R/05_voorspel_doorstroom_huidig.R", encoding = "UTF-8")
list2env(voorspel_doorstroom_huidig(Kliniekset, ICset, condprobs_kl_ic , condprobs_ic_kl, laatste_datum_opnamebestand, aantal_dagen, overplaatsingen_er_uit = overplaatsingen_er_uit), envir = .GlobalEnv)

# Voorspel doorstroom van kliniek naar IC voor toekomstige patiënten
source("R/06_voorspel_doorstroom_toekomst.R", encoding = "UTF-8")
list2env(fn_voorspel_doorstroom_toekomst(condprobs_ic_kl, condprobs_kl_ic, preds_opnames_kl, preds_opnames_ic, aantal_dagen), envir = .GlobalEnv)

# Voorspel survival van toekomstige patiënten
# Tel doorstroom huidig en doorstroom toekomstig op bij instroom
source("R/07_voorspel_bezetting_toekomst.R", encoding = "UTF-8")
list2env(fn_voorspel_bezetting_toekomst(doorstroom_kl_ic_toekomst, doorstroom_ic_kl_toekomst,
                                        preds_opnames_ic, preds_opnames_kl, condprobs_kl,condprobs_ic,
                                        doorstroom_kl_ic_huidig, doorstroom_ic_kl_huidig, aantal_dagen), envir =environment())

# Plots en tabellen genereren
# Koppelen data van instroom, uitstroom, doorstroom
source("R/08_koppel_data.R", encoding = "UTF-8")
list2env(koppel_data(verwachte_bezetting_huidig_kl, verwachte_bezetting_huidig_ic,
                   preds_bezet_toekomst_kl , preds_bezet_toekomst_ic ), envir = environment())
  
plot(preds_bezet_tot_ic$datum, preds_bezet_tot_ic$log, type = 'l', col = 'red')