
get_departure_info = function(condprobs_ic, condprobs_kl, condprobs_ic_kl, condprobs_kl_ic){
  #deze functie haald de nodige info op over ligduurverdelingen voor DES
  
  # functie maken geaggregeerde ligduur op ic
  departure_df_IC <-  condprobs_ic$est %>% 
    select(aantal_dagen_vooruit, ligduur_01) %>% 
    rename(ligduur = aantal_dagen_vooruit, cdf = ligduur_01) %>% 
    mutate(cdf = 1-cdf) %>% add_row(ligduur = 0, cdf = 0, .before = 1)%>%mutate(ligduur = ligduur - 1)
  
  departure_info_ic    = cdf_df_to_fn(departure_df_IC)
  
  # functie maken ligduur patienten die van de ic naar kliniek gaan
  departure_df_IC_KL <- condprobs_ic_kl$est %>% 
    select(aantal_dagen_vooruit, ligduur_01) %>% 
    rename(ligduur = aantal_dagen_vooruit, cdf = ligduur_01) %>%
    add_row(ligduur = 0, cdf = 0, .before = 1)%>%mutate(ligduur = ligduur - 1)
  
  prob_IC_KL = max(departure_df_IC_KL$cdf) # transitiekans
  departure_info_ic_kl = cdf_df_to_fn(departure_df_IC_KL %>% mutate(cdf = cdf/prob_IC_KL))
  
  # functie maken ligduurkans patienten die van de ic niet naar kliniek gaan
  departure_df_IC_rest =merge(departure_df_IC, departure_df_IC_KL, by = 'ligduur', all = TRUE) %>% 
    rename(cdf_tot = cdf.x, cdf_ic_kl = cdf.y) %>% 
    fill(cdf_tot, cdf_ic_kl) %>%
    mutate(cdf_res = floor_at_0(cdf_tot -cdf_ic_kl)) %>%
    select(-cdf_tot, -cdf_ic_kl) %>% 
    mutate(cdf_res = cdf_res/(1-prob_IC_KL))%>%
    rename(cdf = cdf_res)
  
  departure_info_ic_rest = cdf_df_to_fn(departure_df_IC_rest)
  
  # functie maken geaggregeerde ligduurkans op kliniek
  departure_df_KL <-  condprobs_kl$est %>% 
    select(aantal_dagen_vooruit, ligduur_01) %>% 
    rename(ligduur = aantal_dagen_vooruit, cdf = ligduur_01) %>% 
    mutate(cdf = 1-cdf) %>% add_row(ligduur = 0, cdf = 0, .before = 1)%>%mutate(ligduur = ligduur - 1)
  
  
  departure_info_kl    = cdf_df_to_fn(departure_df_KL)
  
  #functie maken ligduurkans patienten die van kliniek naar ic gaan
  departure_df_KL_IC <- condprobs_kl_ic$est %>% 
    select(aantal_dagen_vooruit, ligduur_01) %>% 
    rename(ligduur = aantal_dagen_vooruit, cdf = ligduur_01) %>%
    add_row(ligduur = 0, cdf = 0, .before = 1)%>%mutate(ligduur = ligduur - 1)
  
  prob_KL_IC = max(departure_df_KL_IC$cdf) # transitiekans
  departure_info_kl_ic = cdf_df_to_fn(departure_df_KL_IC %>% mutate(cdf = cdf/prob_KL_IC))
  
  #functie maken ligduurkans patienten die niet van kliniek naar ic gaan
  departure_df_KL_rest =merge(departure_df_KL, departure_df_KL_IC, by = 'ligduur', all = TRUE) %>% 
    rename(cdf_tot = cdf.x, cdf_kl_ic = cdf.y) %>% 
    fill(cdf_tot, cdf_kl_ic) %>%
    mutate(cdf_res = floor_at_0(cdf_tot -cdf_kl_ic)) %>%
    select(-cdf_tot, -cdf_kl_ic) %>% 
    mutate(cdf_res = cdf_res/(1-prob_KL_IC)) %>%
    rename(cdf = cdf_res)
  
  departure_info_kl_rest = cdf_df_to_fn(departure_df_KL_rest)
  
  #alles opslaan in lijst, rds bestand ervan maken
  departure_info = list(prob_KL_IC = prob_KL_IC, prob_IC_KL = prob_IC_KL,
                        departure_info_ic = departure_info_ic, departure_info_kl = departure_info_kl,
                        departure_info_ic_kl = departure_info_ic_kl, departure_info_kl_ic = departure_info_kl_ic,
                        departure_info_ic_rest = departure_info_ic_rest, departure_info_kl_rest = departure_info_kl_rest)
  
  return(departure_info)
}

get_arrival_rates = function(preds_ic, preds_kl, departure_info){
  # deze functie haalt de nodige info op wat betreft aankomstintensiteit voor DES
  
  list2env(departure_info, envir = environment())
  
  start_date = preds_kl$datum[1] - days(1)
  horizons = rep(length(preds_kl$datum), length(colnames(preds_kl))-1)  # horizon in dagen 
  scenarios = colnames(preds_kl%>% select(-datum))
  names(horizons) = scenarios
  
  list_scenario_arrival_rates = list()
  for(scenario in scenarios){
    horizon = horizons[scenario]
    vector_cuml_intensity_IC =  c(0, pull(cumsum(select(preds_ic, all_of(scenario)))))
    vector_cuml_intensity_KL =  c(0, pull(cumsum(select(preds_kl, all_of(scenario)))))
    
    #thining poisson stream by filtering on future path
    vector_cuml_intensity_IC_KL = vector_cuml_intensity_IC*prob_IC_KL
    vector_cuml_intensity_IC_rest = vector_cuml_intensity_IC*(1-prob_IC_KL)
    vector_cuml_intensity_KL_IC = vector_cuml_intensity_KL*prob_KL_IC
    vector_cuml_intensity_KL_rest = vector_cuml_intensity_KL*(1-prob_KL_IC)
    
    #obtaining intensity info from these vectors
    arrival_intensity_info_IC_KL = create_arrival_info(horizon, vector_cuml_intensity_IC_KL)
    arrival_intensity_info_IC_rest = create_arrival_info(horizon, vector_cuml_intensity_IC_rest)
    arrival_intensity_info_KL_IC = create_arrival_info(horizon, vector_cuml_intensity_KL_IC)
    arrival_intensity_info_KL_rest = create_arrival_info(horizon, vector_cuml_intensity_KL_rest)
    
    list_scenario_arrival_rates[[scenario]] = list(KL_IC = arrival_intensity_info_KL_IC,
                                                   KL_rest = arrival_intensity_info_KL_rest,
                                                   IC_KL = arrival_intensity_info_IC_KL,
                                                   IC_rest = arrival_intensity_info_IC_rest)
  }
return(list_scenario_arrival_rates)}

get_patienttype_info = function(departure_info ){
  # deze functie bepaalt de nodige info over vertrekverdelingen voor patiëntklassen 
  list2env(departure_info, envir = environment())

  patienttype_info = list(  # patients at clinic going to the IC and then outside
                            pat_KL_IC   = list(departure_dist_info       = departure_info_kl_ic,
                                               departure_transition_info = c( IC = constantfn(1)),             #ptype always goes to IC
                                               type_transition_info      = c(pat_IC_cfKL = constantfn(1)),     #ptype always becomes patients at IC, which were at clinic
                                               department_arrival_info   = c(Kliniek = 1),                     #ptype always arrives at kliniek
                                               arrival_distribution      = 'NHPois'),
                          
                          #patients going from clinic to outside
                          pat_IC_rest = list(departure_dist_info       = departure_info_ic_rest,              
                                               department_arrival_info   = c( IC = 1),                         #ptype always arrives at IC
                                               arrival_distribution      = 'NHPois'),                      
                          
                          # patients at IC going to the clinic and then outside
                          pat_IC_KL   = list(departure_dist_info       = departure_info_ic_kl,
                                             departure_transition_info = c(Kliniek = constantfn(1)),          #ptype always goes to kliniek
                                             type_transition_info      = c(pat_KL_cfIC = constantfn(1)),      # ptype always becomes patients at clinic, which were at IC
                                             department_arrival_info   = c( IC = 1),                          #ptype always arrives at IC
                                             arrival_distribution      = 'NHPois'),
                          
                          #patients going from clinic to outside
                          pat_KL_rest = list(departure_dist_info       = departure_info_kl_rest,              
                                             department_arrival_info   = c(Kliniek = 1),                     #ptype always arrives at clinic
                                             arrival_distribution      = 'NHPois'),
                          
                          #patients at IC coming from clinic (going to outside)
                          pat_IC_cfKL   = list(departure_dist_info       = departure_info_ic),              #ptype only departs
                          
                          #patients at clinic coming from IC going outside
                          pat_KL_cfIC   = list(departure_dist_info       = departure_info_kl)               #ptype can only depart
                          
  )
  return(patienttype_info)
}



get_info_current_patients = function(ICset, Kliniekset, condprobs_ic_kl, condprobs_kl_ic, condprobs_kl_ic_gbhbp, condprobs_ic, condprobs_kl, overplaatsingen_er_uit = F){
 
  #als overgeplaatste pat (vanuit andere zkh) er uit moeten, stop dan die huidige pat niet in de lijst
  if(overplaatsingen_er_uit){
    res = haal_paden_overplaatsingen_er_uit(Kliniekset, ICset)
    Kliniekset = res[[1]]
    ICset = res[[2]]
  }
  
  # vind opnamemutaties nog aanwezige patiënten
  df_kl_aanwezig = Kliniekset %>% filter(vertrek_type == "Kliniek - Nog opgenomen")
  df_ic_aanwezig = ICset %>% filter(vertrek_type == "IC - Nog opgenomen")
  

  #vind of pat wel/niet behandelbeperking heeft, vind ligduren van de patiënten
  indc_behandelbeperking = !is.na(df_kl_aanwezig$BehandelBeperking) & df_kl_aanwezig$BehandelBeperking != 'Geen behandelbeperking'
  ligduren_kl = df_kl_aanwezig$ligduur
  ligduren_ic = df_ic_aanwezig$ligduur
  names(ligduren_kl) = c()
  names(ligduren_ic) = c()

  #current information  for all patients currently in the system (can be patient-specific)
  current_patient_info = append(lapply(seq_len(length(ligduren_kl)), function(i){list(LOS_init = ligduren_kl[i],
                                                                                      department = 'Kliniek',
                                                                                      prob_types = if(indc_behandelbeperking[i]){
                                                                                        list(pat_KL_IC = 0, pat_KL_rest = 1)
                                                                                      }else{list(pat_KL_IC = condprob_transition(condprobs_kl_ic_gbhbp$est, condprobs_kl$est, ligduren_kl[i]),
                                                                                                 pat_KL_rest = 1-condprob_transition(condprobs_kl_ic_gbhbp$est, condprobs_kl$est, ligduren_kl[i]))})}),



                                lapply(seq_len(length(ligduren_ic)),function(i){list(LOS_init = ligduren_ic[i],
                                                                                     department = 'IC',
                                                                                     prob_types = list(pat_IC_KL = condprob_transition(condprobs_ic_kl$est, condprobs_ic$est, ligduren_ic[i]),
                                                                                                       pat_IC_rest = 1-condprob_transition(condprobs_ic_kl$est,condprobs_ic$est, ligduren_ic[i])))}))


return(mget(c('ligduren_ic', 'ligduren_kl', 'current_patient_info')))}

bereken_historische_bezetting = function(Kliniekset, ICset, ligduren_ic, ligduren_kl, voorspeldatum, overplaatsingen_er_uit = F){
  
  #als overgeplaatste pat (vanuit andere zkh) er uit moeten, haal die patiënten dan van de bezetting af
  if(overplaatsingen_er_uit){
    list2env(haal_paden_overplaatsingen_er_uit(Kliniekset, ICset), envir = environment())
     }
  
  #determining the starting dates of the historical measurements, and endpoint (which is starting point predicitons)
  startdatum_gesch = floor_date(as.POSIXct(min(c(min(Kliniekset$startdatumtijd), min(ICset$startdatumtijd)))), unit = 'days')
  einddatum_gesch = as.POSIXct(voorspeldatum)
  
  # datums_historie = seq(startdatum_gesch, einddatum_gesch, by = 'mins')
  datums_historie = seq(startdatum_gesch, einddatum_gesch, by = 'hours')
  
  Kliniekset$einddatumtijd[is.na(Kliniekset$einddatumtijd)] = einddatum_gesch + days(1)
  bezetting_kl_minute = apply(sapply(seq_len(nrow(Kliniekset)), function(i){datums_historie > Kliniekset$startdatumtijd[i] &datums_historie <= Kliniekset$einddatumtijd[i] }), 1, sum)
  
  ICset$einddatumtijd[is.na(ICset$einddatumtijd)] = einddatum_gesch + days(1)
  bezetting_ic_minute = apply(sapply(seq_len(nrow(ICset)), function(i){datums_historie > ICset$startdatumtijd[i] &datums_historie <= ICset$einddatumtijd[i] }), 1, sum)
  
  # creating tibble with historical occupation
  hist_bezetting_kl = tibble(datum = datums_historie, bezetting = bezetting_kl_minute) %>% 
    mutate(datum = floor_date(datum, unit = 'days')) %>% 
    group_by(datum)%>% 
    summarise(bezetting = max(bezetting), .groups = 'drop')
  
  hist_bezetting_kl$bezetting[dim(hist_bezetting_kl)[[1]]] = length(ligduren_kl)# making last element overlap with predictions
  
  hist_bezetting_ic = tibble(datum = datums_historie, bezetting = bezetting_ic_minute )%>% 
    mutate(datum = floor_date(datum, unit = 'days')) %>% 
    group_by(datum)%>% 
    summarise(bezetting = max(bezetting), .groups = 'drop')
  hist_bezetting_ic$bezetting[dim(hist_bezetting_ic)[[1]]] = length(ligduren_ic) # making last element overlap with predictions
  
  #opslaan
  einddatum = as.Date(einddatum_gesch + days(1)) 
  
  return(mget(c('hist_bezetting_ic', 'hist_bezetting_kl')))
}

get_system_information = function(Kliniekset, ICset, preds_opnames_ic, preds_opnames_kl, condprobs_ic, condprobs_kl, condprobs_ic_kl, condprobs_kl_ic, condprobs_kl_ic_gbhbp, voorspeldatum,
                                  overplaatsingen_er_uit = F){
  
  departure_info              = get_departure_info(condprobs_ic, condprobs_kl, condprobs_ic_kl, condprobs_kl_ic)
  list_scenario_arrival_rates = get_arrival_rates(preds_opnames_ic, preds_opnames_kl, departure_info)
  patienttype_info            = get_patienttype_info(departure_info)
  list2env(get_info_current_patients(ICset, Kliniekset, condprobs_ic_kl , condprobs_kl_ic, condprobs_kl_ic_gbhbp, condprobs_ic, condprobs_kl, overplaatsingen_er_uit = overplaatsingen_er_uit ), envir = environment())
  list2env(bereken_historische_bezetting(Kliniekset, ICset, ligduren_ic = ligduren_ic, ligduren_kl = ligduren_kl, voorspeldatum, overplaatsingen_er_uit = overplaatsingen_er_uit ), envir = environment())
  return(mget(c('departure_info', 'list_scenario_arrival_rates', 'patienttype_info', 'current_patient_info','hist_bezetting_ic', 'hist_bezetting_kl')))
}

