
plot_voorspelde_benutting = function(pred_df_dept, dept, ziekenhuis,scenarios, CI_val = 0.8  ){
  #genereert plot van de benutting per dag + confidence interval
  for(scenario in scenarios){
    pred_df_dept = pred_df_dept %>% rename(!!paste0('Voorspelling ', names_cols[scenario]) := paste0('avg_', scenario))
    pred_df_dept = pred_df_dept %>% rename(!!paste0('Bovengrens 80% predictie interval ', names_cols[scenario]) := paste0('ubCI80%_', scenario))
    pred_df_dept = pred_df_dept %>% rename(!!paste0('Onder 80% predictie interval ', names_cols[scenario]) := paste0('lbCI80%_', scenario))
  }
  
  
  plot_voorspelling_dept = ggplot(pred_df_dept, aes(x = datum)) +
    ylim(0, NA) +
    xlab("Datum") +
    ylab(paste0("Bedbenutting", dept)) +
    ggtitle(
      paste0('Voorspelde Bedbenutting ',dept,' & ',toString(round(CI_val*100)),'% bovengrens', " - ", naam_voluit))+
    geom_line(aes(y = bezetting), size = 1.2 ) +
    scale_color_manual(name = '', values = colors)+
    scale_x_date(breaks = unique(round_date(seq(min(pred_df_dept$datum), max(pred_df_dept$datum), by = 
                                                  round(difftime(max(pred_df_dept$datum), min(pred_df_dept$datum))/10)), unit = 'days')), labels = date_format( "%d -%m"))
  
  for(scenario in scenarios){
    plot_voorspelling_dept = plot_voorspelling_dept + 
      geom_line(aes_string(y = pred_df_dept[,paste0("Voorspelling ", names_cols[scenario])], color = shQuote(names_cols[scenario])),size = 1.2)+
      geom_ribbon(aes_string(ymin = pred_df_dept[,paste0("Voorspelling ", names_cols[scenario])], ymax = pred_df_dept[,paste0('Bovengrens 80% predictie interval ', names_cols[scenario])]),
                  alpha =0.2, fill =  colors[names_cols[scenario]] )
  }
  
  return(ggplotly(plot_voorspelling_dept))
}

plot_kwantielen_maxima = function(maxima_dept, dept, naam_voluit){
  #genereert een plot van de verdeling van de maximale bezetting
  pvals = c(0.5,0.6,0.7,0.8,0.9)
  qvals = as_tibble(apply(maxima_dept, 2, function(x){quantile(x,pvals)})) %>% add_column(probs = pvals, .before = 1)
  num_scen = dim(qvals)[[2]]-1
  plot_maxima = qvals%>% mutate_if(is.numeric, round, 2) %>% pivot_longer(-probs) %>%
    ggplot(aes(x = probs, color = name)) +
    geom_line(aes(y = value), size = 0.8)+
    geom_point(aes(y = value))+
    facet_wrap(~name, scale = 'free')+
    labs(y = 'Gedekte Maximale Bedbenutting', x = 'Overschrijdingskans' ) +
    scale_color_discrete(name = '')+
    scale_y_continuous(breaks =  unique(unlist(apply(maxima_dept,2,unique))))+
    scale_x_continuous(breaks = pvals, labels = percent)+
    geom_segment(data=qvals%>% pivot_longer(-probs), aes(y = value, xend=-Inf, yend=value), linetype = 'dashed') +
    geom_segment(data=qvals%>% pivot_longer(-probs), aes(xend=probs, yend=-Inf, y = value), linetype = 'dashed')+
    add_title_DES(paste0('Kwantielen Maximaal # Patiënten ', dept),naam_voluit)
  return(ggplotly(plot_maxima))
}

plot_voorspelling_aankomsten = function(voorspelde_opnames, naam_voluit, scenarios){
  # plot voorspelde opnames per dag + historie
  if('log_upr' %in% colnames(voorspelde_opnames)){
    #genereert een plot van totale aankomsten systeem
    voorspelde_opnames[, 'log_upr_pred'] = voorspelde_opnames[, 'log_upr']
    voorspelde_opnames[!is.na(voorspelde_opnames$nieuwe_opn), 'log_upr_pred'] = NA}
  
  plot_nieuwe_opnames_tot <- voorspelde_opnames%>% mutate_if(is.numeric, round, 2)
  plot_nieuwe_opnames_tot = plot_nieuwe_opnames_tot %>% select(c('datum',"Geobserveerd" = nieuwe_opn, all_of(scenarios)))
  for(scenario in scenarios){
    plot_nieuwe_opnames_tot = plot_nieuwe_opnames_tot %>% rename( !!names_cols[scenario]:= scenario)
  }
  
  plot_nieuwe_opnames_tot=plot_nieuwe_opnames_tot %>% 
    pivot_longer(-datum) %>%
    ggplot() +
    # Predictie-interval
    geom_ribbon(aes(datum, ymin = `80% ondergrens model`, ymax = `80% bovengrens model`),
                data = voorspelde_opnames %>% rename(`80% bovengrens model` = log_upr,
                                                 `80% ondergrens model` = log_lwr), fill = "blue", alpha = .1
    ) +
    # Voorspelling
    geom_line(aes(datum, value, col = name), size = 1.2) +
    # scale_color_discrete(labels = var_namen, name = NULL) +
    xlab("Datum") +
    ylab("Directe opnames") +
    add_title_DES("Voorspelde directe opnames", naam_voluit) +
    theme(legend.title = element_blank())
  
  return(ggplotly(plot_nieuwe_opnames_tot))
}

plot_voorspelde_aankomsten_dept = function(nieuwe_opnames_dept, name_dept, naam_voluit, scenarios){
  # genereert een plot van de aankomsten voor een afdeling + voorspellingen
  #genereert een plot van totale aankomsten systeem
  if('log_upr' %in% colnames(nieuwe_opnames_dept)){
    #genereert een plot van totale aankomsten systeem
    nieuwe_opnames_dept[, 'log_upr_pred'] = nieuwe_opnames_dept[, 'log_upr']
    nieuwe_opnames_dept[!is.na(nieuwe_opnames_dept$nieuwe_opn), 'log_upr_pred'] = NA}
  
  plot_voorspelde_opnames_dept <- nieuwe_opnames_dept%>% mutate_if(is.numeric, round, 2)
  plot_voorspelde_opnames_dept = plot_voorspelde_opnames_dept %>% select(c('datum',"Geobserveerd" = opnames, all_of(scenarios)))
  for(scenario in scenarios){
    plot_voorspelde_opnames_dept = plot_voorspelde_opnames_dept %>% rename( !!names_cols[scenario]:= scenario)
  }
  
  
  plot_voorspelde_opnames_dept= plot_voorspelde_opnames_dept  %>%
    pivot_longer(-datum) %>%
    ggplot() +
    # Voorspelling
    geom_line(aes(datum, value, col = name), size = 1.2) +
    scale_color_discrete(name = NULL) +
    xlab("Datum") +
    ylab("Nieuwe opnames")+
    add_title_DES(paste0("Voorspelde directe opnames ", name_dept), naam_voluit)
  
  return(ggplotly(plot_voorspelde_opnames_dept))
}

plot_transitiekans_afdelingen = function(condprobs_transitie, naam_transitie, naam_voluit){
  # genereert een plot van de kans op een transitie tussen afdelingen, afhankelijk van de ligduur
  plot_transitiekans_afd <- ggplot(
    condprobs_transitie$est,
    aes(aantal_dagen_vooruit, ligduur_01)
  ) +
    geom_step() +
    scale_x_continuous(minor_breaks = 0:max(condprobs_transitie$est$aantal_dagen_vooruit)) +
    xlab("Ligduur T (dagen)") +
    ylab(paste0("Kans op doorstroom ",naam_transitie," vóór ligduur T")) +
    add_title_DES(paste0("Empirische kansverdeling doorstroom ", naam_transitie ), naam_voluit)
  return(plot_transitiekans_afd)
}

plot_ligduurverdeling_afdeling = function(survival_dept, data_dept, name_dept, naam_voluit){
  # maakt een plot van de empirische ligduurverdeling, op basis van de Kaplan-Meijer schatter
  kaplan_meijer <-
    ggsurvplot(survival_dept,
               legend = 'none',
               censor = F,
               surv.median.line = "hv",
               data = data_dept, pval = FALSE, title =
                 paste0("Ligduurverdeling ",name_dept," - ", naam_voluit,
                        '    mediaan = ', surv_median(survival_dept)$median, ' dagen '),
               xlab = 'ligduur T (dagen)',
               ylab = 'kans dat patiënt er nog ligt na ligduur T',
               legend.title=""
    ) 
  return(kaplan_meijer)
}

plot_pat_per_ld_dept = function(pat_per_ld_dept, name_dept, naam_voluit){
  # maakt een plot van het aantal patiënten per ligduur bij een afdeling
  plot_patienten_per_ligduur_dept <- pat_per_ld_dept %>%
    pivot_longer(everything(), names_to = "ligduur", values_to = "aantal") %>%
    mutate(ligduur = 1:nrow(.)) %>%
    ggplot(aes(ligduur, aantal)) +
    geom_bar(stat = "identity") +
    xlab("Ligduur (in dagen)") +
    ylab("Aantal Patiënten") +
    scale_y_continuous(breaks = seq(0, max(pat_per_ld_dept)))+
    add_title_DES(paste0("Aantal opgenomen patiënten per ligduur ", name_dept), naam_voluit)
  return(ggplotly(plot_patienten_per_ligduur_dept))
}

plot_leegloop_huidige_pat = function(verwachte_bezetting_huidig_dept, name_dept, ziekenhuis){
  # maakt een plot van het aantal verwachte patiënten op de afdeling gegeven dat er geen instroom meer is.
  plot_verwachte_bezetting_huidig_dept <- verwachte_bezetting_huidig_dept%>%
    mutate_if(is.numeric, round, 2) %>%
    select(datum, som) %>% rename(`aantal patiënten` =  som)%>%
    ggplot(aes(datum, `aantal patiënten`)) +
    geom_line() +
    xlab("Datum") +
    ylab("Benutting bedden") +
    add_title_DES(paste0("Voorspelde bedbenutting huidige patiënten ", name_dept), ziekenhuis)+
    scale_y_continuous(breaks = seq(round(min(verwachte_bezetting_huidig_dept$som)),
                                    round(max(verwachte_bezetting_huidig_dept$som)),
                                    by =  ceil((max(verwachte_bezetting_huidig_dept$som) - min(verwachte_bezetting_huidig_dept$som))/10)))
  return(ggplotly(plot_verwachte_bezetting_huidig_dept))
}

make_plots = function(ziekenhuis, pred_df_ic, pred_df_kl, hist_bezetting_ic, hist_bezetting_kl, maxima_IC, maxima_kl,
                      voorspelde_opnames, df_nieuwe_opnames, preds_opnames_ic, preds_opnames_kl,
                      condprobs_ic_kl, condprobs_kl_ic, fit_kl, fit_ic, naam_voluit, ICset, Kliniekset,
                      pat_per_ld_ic, pat_per_ld_kl, verwachte_bezetting_huidig_ic, verwachte_bezetting_huidig_kl, scenarios,
                      laatste_datum_opnamebestand,opslaan = T, plot = F, opslaan_voorspellingen = T){
  #basisfunctie om plots te maken
  
  #plot verwachte bezetting IC
  hist_bezetting_ic$datum = as.Date(hist_bezetting_ic$datum)
  pred_df_ic_aug = merge(pred_df_ic, hist_bezetting_ic,  by = 'datum', all = TRUE)
  
    plot_voorspelling_ic_uitgezoomd = plot_voorspelde_benutting(pred_df_ic_aug, 'IC', ziekenhuis, scenarios)

  #plot verwachte bezetting kliniek
  hist_bezetting_kl$datum = as.Date(hist_bezetting_kl$datum)
  pred_df_kl_aug = merge(pred_df_kl, hist_bezetting_kl, by = 'datum', all = TRUE)
  plot_voorspelling_kl_uitgezoomd = plot_voorspelde_benutting(pred_df_kl_aug, 'Kliniek', ziekenhuis, scenarios)
  
  #plot maxima IC
  plot_maxima_IC = plot_kwantielen_maxima(maxima_IC, 'IC', naam_voluit)
  
  #plot maxima kliniek
  plot_maxima_kliniek = plot_kwantielen_maxima(maxima_kl, 'Kliniek', naam_voluit)
  
  # plot voorspelde opnames + observaties
  plot_nieuwe_opnames_tot = plot_voorspelling_aankomsten(voorspelde_opnames, naam_voluit, scenarios)
  
  # plot voorspelde opnames + observaties IC
  nieuwe_opnames_ic <- df_nieuwe_opnames %>%
    select(datum, aantal_ic) %>%
    rename(opnames = aantal_ic) %>%
    merge(y = preds_opnames_ic, by = "datum", all = TRUE)
  
  plot_nieuwe_opnames_ic = plot_voorspelde_aankomsten_dept(nieuwe_opnames_ic, 'IC', naam_voluit, scenarios) 
  
  # plot voorspelde opnames + observaties Kliniek
  nieuwe_opnames_kl <- df_nieuwe_opnames %>%
    select(datum, aantal_kliniek) %>%
    rename(opnames = aantal_kliniek) %>%
    merge(y = preds_opnames_kl, by = "datum", all = TRUE)
  
  plot_nieuwe_opnames_kl = plot_voorspelde_aankomsten_dept(nieuwe_opnames_kl, 'Kliniek', naam_voluit, scenarios) 
  
  #plot transitiekans van kliniek naar IC
  kaplan_meijer_kl_ic = plot_transitiekans_afdelingen(condprobs_kl_ic, 'KL - IC', naam_voluit)
  
  #plot transitiekans van IC naar kliniek
  kaplan_meijer_ic_kl = plot_transitiekans_afdelingen(condprobs_ic_kl, 'IC - KL', naam_voluit)
  
  #plot ligduurverdeling Kliniek
  kaplan_meijer_kl = plot_ligduurverdeling_afdeling(fit_kl, Kliniekset, 'Kliniek', naam_voluit)
  
  #plot ligduurverdeling IC
  kaplan_meijer_ic = plot_ligduurverdeling_afdeling(fit_ic, ICset, 'IC', naam_voluit)
  
  #plot aantal patiënten per ligduur IC
  plot_patienten_per_ligduur_ic = plot_pat_per_ld_dept(pat_per_ld_ic, 'IC', naam_voluit )
  
  #plot aantal patiënten per ligduur Kliniek
  plot_patienten_per_ligduur_kl = plot_pat_per_ld_dept(pat_per_ld_kl, 'Kliniek', naam_voluit )
  
  #plot de verwachte leegloop van huidige patiënten in de kliniek
  plot_verwachte_bezetting_huidig_kl = plot_leegloop_huidige_pat(verwachte_bezetting_huidig_kl, 'Kliniek', naam_voluit)
  
  #plot de verwachte leegloop van huidige patiënten op de IC
  plot_verwachte_bezetting_huidig_ic = plot_leegloop_huidige_pat(verwachte_bezetting_huidig_ic, 'IC', naam_voluit)
  
  
  if(plot){ # als plot == T dan wordt alles geplot als deze functie aangeroepen wordt
    print(plot_voorspelling_kl_uitgezoomd, ziekenhuis)
    print(plot_voorspelling_ic_uitgezoomd, ziekenhuis)
    print(plot_maxima_kliniek, ziekenhuis)
    print(plot_maxima_IC, ziekenhuis)
    print(plot_nieuwe_opnames_tot, ziekenhuis)
    print(plot_nieuwe_opnames_ic, ziekenhuis)
    print(plot_nieuwe_opnames_kl, ziekenhuis)
    print(kaplan_meijer_kl_ic, ziekenhuis)
    print(kaplan_meijer_ic_kl, ziekenhuis)
    print(kaplan_meijer_kl, ziekenhuis)
    print(kaplan_meijer_ic, ziekenhuis)
    print(plot_patienten_per_ligduur_ic, ziekenhuis)
    print(plot_patienten_per_ligduur_kl, ziekenhuis)
    print(plot_verwachte_bezetting_huidig_kl, ziekenhuis)
    print(plot_verwachte_bezetting_huidig_ic, ziekenhuis)
  }
  
  if(opslaan){ # als opslaan == T dan wordt alles geplot als deze functie aangeroepen wordt
  schrijf_rds(plot_voorspelling_kl_uitgezoomd, ziekenhuis)
  schrijf_rds(plot_voorspelling_ic_uitgezoomd, ziekenhuis)
  schrijf_rds(plot_maxima_kliniek, ziekenhuis)
  schrijf_rds(plot_maxima_IC, ziekenhuis)
  schrijf_rds(plot_nieuwe_opnames_tot, ziekenhuis)
  schrijf_rds(plot_nieuwe_opnames_ic, ziekenhuis)
  schrijf_rds(plot_nieuwe_opnames_kl, ziekenhuis)
  schrijf_rds(kaplan_meijer_kl_ic, ziekenhuis)
  schrijf_rds(kaplan_meijer_ic_kl, ziekenhuis)
  schrijf_rds(kaplan_meijer_kl, ziekenhuis)
  schrijf_rds(kaplan_meijer_ic, ziekenhuis)
  schrijf_rds(plot_patienten_per_ligduur_ic, ziekenhuis)
  schrijf_rds(plot_patienten_per_ligduur_kl, ziekenhuis)
  schrijf_rds(plot_verwachte_bezetting_huidig_kl, ziekenhuis)
  schrijf_rds(plot_verwachte_bezetting_huidig_ic, ziekenhuis)}
  
  if(opslaan_voorspellingen){
    #sla voorspellingen op als .csv
    write_csv(pred_df_kl_aug %>% select(datum, `voorspelling Richards curve` = avg_log, `Betrouwbaarheids Bovengrens voorspelling` = `ubCI80%_log`, `Betrouwbaarheids Ondergrens voorspelling` = `lbCI80%_log`,
                                        bedbenutting = bezetting), paste0('predictions/voorspelde_bedbenutting_Kliniek_',ziekenhuis,'.csv'))
    write_csv(pred_df_ic_aug %>% select(datum, `voorspelling Richards curve` = avg_log, `Betrouwbaarheids Bovengrens voorspelling` = `ubCI80%_log`,`Betrouwbaarheids Ondergrens voorspelling` = `lbCI80%_log`,
                                        bedbenutting = bezetting), paste0('predictions/voorspelde_bedbenutting_IC_', ziekenhuis,'.csv'))
    
    #sla opnames totaal op
    write_csv(voorspelde_opnames %>% select(datum, `Opnames Ziekenhuis` = nieuwe_opn, `Ondergrens Voorspelde Opnames` = log_lwr, `Voorspelde Opnames` = log, `Bovengrens Voorspelde Opnames` = log_upr),
              paste0('predictions/voorspelde_opnames_ziekenhuis_',ziekenhuis,'.csv'))
    
    #sla opnames IC op
    write_csv(nieuwe_opnames_ic %>% select(datum, `Opnames Ziekenhuis` = opnames, `Ondergrens Voorspelde Opnames` = log_lwr, `Voorspelde Opnames` = log, `Bovengrens Voorspelde Opnames` = log_upr),
              paste0('predictions/voorspelde_opnames_IC_ziekenhuis_',ziekenhuis,'.csv'))
    #sla opnames Kliniek op
    write_csv(nieuwe_opnames_kl %>% select(datum, `Opnames Ziekenhuis` = opnames, `Ondergrens Voorspelde Opnames` = log_lwr, `Voorspelde Opnames` = log, `Bovengrens Voorspelde Opnames` = log_upr),
              paste0('predictions/voorspelde_opnames_Kliniek_ziekenhuis_',ziekenhuis,'.csv'))
    
    
  }
  }