

perform_simulation_study = function(voorspeldatum, aantal_dagen, list_scenario_arrival_rates, departure_info, current_patient_info,
                                    patienttype_info, scenarios, print_ = T, M = 1000, until_convg = F){ # M ~number of simulations
  
  #determining horizons and scenarios for simulation
  start_date = as.Date(voorspeldatum)
  horizon    = aantal_dagen # horizon in dagen
  horizons = rep(horizon, length(scenarios))
  names(horizons) = scenarios
  department_names = c('Kliniek', 'IC')
  
  #performing simulation studies for all scenario's
  cores <- detectCores(logical = FALSE)
  if(print_){print('Performing simstudy:')}
  simresults = list()
  for(scenario in scenarios){
    if(print_){print(paste0(' huidig scenario: ', scenario))}
    
    #if the scenario is opnames_nul_notrans, we do not allow transitions and arrivals, in this if-statement the transition probabilities are set to 0
    if(scenario == 'opnames_nul_notrans'){# if we are in situation where no transitions are allowed, set transition probs to 0 
      patienttype_info$pat_KL_IC$departure_transition_info = c(Kliniek = constantfn(0))
      patienttype_info$pat_IC_KL$departure_transition_info = c(Kliniek = constantfn(0))
    }
    
    #setting arrival rates
    patienttype_info$pat_KL_IC$arrival_info = list_scenario_arrival_rates[[scenario]]$KL_IC
    patienttype_info$pat_KL_rest$arrival_info = list_scenario_arrival_rates[[scenario]]$KL_rest
    patienttype_info$pat_IC_KL$arrival_info = list_scenario_arrival_rates[[scenario]]$IC_KL
    patienttype_info$pat_IC_rest$arrival_info = list_scenario_arrival_rates[[scenario]]$IC_rest
    
    # perform DES in parallel (1000 reps)
    flag = F
    simres = list()
    while(!flag){
      cl <- makeCluster(cores)
        registerDoParallel(cl, cores=cores)
        perform_DES(horizons[scenario] , department_names , patienttype_info , current_patient_info  , start_date = start_date)
        simres_partial = foreach(icount(M), .combine = append, .export = c('perform_DES', 'rnhpois', 'sample_remaining_LOS',
                                                                   'first'), .packages = c('pracma','lubridate','dbplyr','tibble','tidyr','dplyr')) %dopar% {
          list(perform_DES(horizons[scenario] , department_names , patienttype_info , current_patient_info  , start_date = start_date))}
        stopCluster(cl)
    
      simres = append(simres, simres_partial)
      if(until_convg){
        bezetting_ic_scen = as.numeric(Reduce(cbind , lapply(simres, function(tbl){tbl$IC}))[aantal_dagen+1,])
        bezetting_kl_scen = as.numeric(Reduce(cbind , lapply(simres, function(tbl){tbl$Kliniek}))[aantal_dagen+1,])
        flag = sd(boot(bezetting_ic_scen, function(v,i){quantile(v[i],0.025)}, 1000)$t) <=0.5& 
          sd(boot(bezetting_ic_scen, function(v,i){quantile(v[i],0.975)}, 1000)$t)<=0.5&
          confradius(boot(bezetting_ic_scen, function(v,i){mean(v[i])}, 1000)$t)<0.25&
          sd(boot(bezetting_kl_scen, function(v,i){quantile(v[i],0.025)}, 1000)$t) <=0.5& 
          sd(boot(bezetting_kl_scen, function(v,i){quantile(v[i],0.975)}, 1000)$t)<=0.5&
          confradius(boot(bezetting_kl_scen, function(v,i){mean(v[i])}, 1000)$t)<0.25
        M = M * 2
        print(c(sd(boot(bezetting_ic_scen, function(v,i){quantile(v[i],0.025)}, 1000)$t), 
                sd(boot(bezetting_ic_scen, function(v,i){quantile(v[i],0.975)}, 1000)$t),
                confradius(boot(bezetting_ic_scen, function(v,i){mean(v[i])}, 1000)$t),
                sd(boot(bezetting_kl_scen, function(v,i){quantile(v[i],0.025)}, 1000)$t),
                sd(boot(bezetting_kl_scen, function(v,i){quantile(v[i],0.975)}, 1000)$t),
                confradius(boot(bezetting_kl_scen, function(v,i){mean(v[i])}, 1000)$t)
        ))
        print(M)
        }else{
          flag = T
        }
    }
    
    
    # storing simulation results
    simresults[[scenario]] = simres
  }
  return(simresults)
}
