package_list = c('boot','minpack.lm','plotly', 'parallel', 'pracma', 'padr', 'dplyr', 'doParallel', 'Hmisc', 'ggplot2', 'tibble', 'purrr', 'pander', 'scales',
  'tidyverse', 'lubridate','investr','FlexParamCurve', 'survival', 'survminer', 'readr', 'readxl', 'tictoc', 'tibble', "grid", 'naniar'
)

for(package_name in package_list){
  if(!(package_name %in% rownames(installed.packages()))){
    install.packages(package_name, dependencies = T)
  }
  do.call(library, list(package = package_name, character.only = TRUE))}
library(boot)
library(plotly)
library('parallel')
library('pracma')
library('padr')
library('dplyr')
library('doParallel')
library('Hmisc')
library('ggplot2') 
library('tibble') 
library('purrr')
library('pander') 
library('scales')
library('tidyverse')
library('minpack.lm')
library('lubridate')
library('investr')
library('FlexParamCurve') 
library('survival')
library('survminer') 
library('readr')
library('readxl') 
library('tictoc')
library(tibble)
library(grid)

replace_null_with_na = function(v){
  v[strcmp(v,'NULL')] = NA
  return(v)
}

#pad naar pandoc instellen
path = Sys.getenv("RSTUDIO_PANDOC")
Sys.setenv(RSTUDIO_PANDOC=path)

cdf_df_to_fn = function(df){
  #input : df with column ligduur and column cdf at ligduur, output: list of cdf and icdf functions
  return(list(cdf = get_simple_function(df$ligduur, df$cdf),
              icdf = get_inv_simple_fn(df$ligduur, df$cdf)))
}

maak_partiele_set = function(dept_set, date, name_dept, name_other_dept){
  #muteer een kliniekset/ICset zodanig dat alleen de beschikbare data vóór "date" meegenomen is.  
  
  dept_set = dept_set[as.POSIXct(dept_set$startdatumtijd, tz = 'UTC') <= as.POSIXct(date, tz = 'UTC'), ] # haal aankomsten na date weg
  for(i in seq(nrow(dept_set))){
    #als pat aangekomen is vóór date en weggaat na "date", zet einddatumtijd naar "date" en geef aan dat patiënt toen nog lag
    if(  is.na(dept_set[i,]$einddatumtijd)|dept_set[i,]$einddatumtijd > as.POSIXct(date)){
      dept_set[i,]$einddatumtijd = as.POSIXct(date, tz = 'UTC')
      dept_set[i, ]$einddatum = date
      dept_set[i, ]$vertrek_type = paste0(name_dept, ' - Nog opgenomen')
    }
  }
  dept_set$ligduur = sapply(difftime(dept_set$einddatum , dept_set$startdatum, unit = 'days') + 1, as.integer)  # obtaining LOS
  
  #redetermining event variable for survival analysis
  dept_set$event = ifelse(dept_set$vertrek_type == paste0(name_dept, " - ander ziekenhuis"), 0,
                          ifelse(dept_set$vertrek_type == paste0(name_dept, " - Nog opgenomen"), 0, 1))
  return(dept_set)
}

confradius = function(v, level = 0.95){
  return(quantile(v, 1/2+level/2)[[1]] - quantile(v, 1/2 - level/2)[[1]])
}

haal_paden_overplaatsingen_er_uit = function(Kliniekset, ICset){
  colname_pat_nrs = colnames(Kliniekset)[1] # dit is niet bij elk ziekenhuis hetzelfde volgens mij
  
  #vind nummers pat die een keer overgeplaatst zijn
  pat_nr_overgeplaatst = c(Kliniekset %>% filter(Herkomst == 'Ander ziekenhuis')%>% pull(colname_pat_nrs), ICset %>% filter(Herkomst == 'Ander ziekenhuis')%>% pull(colname_pat_nrs)) 
  
  #haal ze uit kliniekset/icset
  Kliniekset = Kliniekset[!is.element(Kliniekset %>% pull(colname_pat_nrs),  pat_nr_overgeplaatst), ]
  ICset = ICset[!is.element(ICset %>% pull(colname_pat_nrs),  pat_nr_overgeplaatst), ]
  
  return(mget(c('Kliniekset', 'ICset')))
}



rnhpois = function(t, cuml_intensity, inv_cuml_intensity){
  # sample the next nonhomogeneous Poisson arrival time at time t given the cumulative intensity function and the inverse
  # cumulative intensity function. Theory is described in 
  # Daley, D. J., & Vere-Jones, D. (2007). An introduction to the theory of point processes: volume II: general theory and structure. Springer Science & Business Media.
  # Basically, when transforming the time axis with the cumulative intensity Lambda(s) = int_0^s lambda(u)du, the resulting process on this time axis is 
  # a unit-rate homogeneous PP. Hence, we sample an unit rate homogeneous PP on this time scale by transforming the last event-time with Lambda, incrementing it with
  # an exponential random variable, and taking the inverse of Lambda of the result.
  
  result = tryCatch(
    {ttime = rexp(1) + cuml_intensity(t)
    if(ttime > cuml_intensity(Inf)){
      return(Inf)
    }else{
       result = inv_cuml_intensity(ttime)
    }},
   warning = function(cond){return(Inf)}
  )

    return(result)}

first = function(v){
  return(v[1])
}


add_title_DES <- function(title, naam_voluit) {
  ggtitle(
    paste0(title, " - ", naam_voluit)
  )
}

laad_rds_DES <- function(naam, ziekenhuis = "etz") {
  if (basename(getwd()) == "rmarkdown") {
    map <- "../DES/rds/"
  } else {
    map <- "DES/rds/"
  }
  
  bestand <- paste0(map, ziekenhuis, "_", naam, ".rds")
  
  read_rds(bestand)
}

colors <- c("Model (Richards curve)" = rgb(0, 176/(176+246),246/(176+246)),
            "Exponentieel" = rgb(163/(165+163), 165/(163+165),0),
            "Constante Instroom" = 'purple',
            "Bovengrens Model (Richards curve)" = 'red',
            'Tweede Piek' = 'blue')

names_cols <- c('log' = "Model (Richards curve)",
              'log_upr' = "Bovengrens Model (Richards curve)",
              'vlak' = 'Constante Instroom',
              'piek' = 'Tweede Piek',
              'exp' = 'Exponentieel')

schrijf_rds_DES <- function(object, ziekenhuis) {
  bestand <- paste0("DES/rds/", ziekenhuis, "_", deparse(substitute(object)), ".rds")
  
  write_rds(object, bestand)
}

floor_at_0 = Vectorize(function(x){max(x,0)})

rewrite_date = Vectorize(function(datestring){
  # rewrites date in format d(d)-m(m)-yyyy hh:mm in the format yyyy-mm-dd hh:mm
  
  #split the string such that we obtain vector c(d(d), m(m), yyyy, hh:mm)
  splitted_string = str_split(datestring, '-')
  splitted_string = c(splitted_string[[1]][1],splitted_string[[1]][2] , unlist(str_split(splitted_string[[1]][3], ' ')))
  
  #adding zero to day number if it only has one number
  if(nchar(splitted_string[1]) == 1){
    splitted_string[1] = paste0('0', splitted_string[1])
  }
  
  #adding zero to month number if it only has one number
  if(nchar(splitted_string[2]) == 1){
    splitted_string[2] = paste0('0', splitted_string[2])
  }
  
  return(paste0(splitted_string[3],'-',splitted_string[2],'-',splitted_string[1],' ', splitted_string[4],':00'))
  
})


Poisregr_logistic = function(dates, counts){
  # fit and return a 4-parameter Richards curve using Poisson regression on count data 
  # input should be the dates and counts on these dates
  
  der_logistic = function(t, A,k,i){
    #returns the derivative of a 3 param logistic curve at time point t (integer) RC(t) =  A / (1+  exp(-k (t-i)))
    return((A*k*exp(-k*(t - i)))/(1 + exp(-k*(t - i)))^2)
  }
  negloglik_logistic = function(datenum,counts, A,k,i){
    #negative Poisson log likelihood where the expected value is a 4 param RC. datenum are the date numbers (integers)
    expectedcounts = der_logistic(datenum, A,k,i)
    return(sum(expectedcounts) - log(expectedcounts)%*%counts)
  }
  
  #calculate date numbers (sort the dates first and counts accordingly)
  counttbl = tibble(dates = dates, counts = counts) %>% arrange(dates) 
  dates = counttbl$dates
  counts = counttbl$counts
  datenum = c(1,1+cumsum(sapply(diff(dates), as.integer))) #numbering dates
  
  #define optimization scheme
  init = c(1,1,1)
  fopt = function(params){
    negloglik_logistic(datenum, counts, params[1] , params[2], params[3])
  }
  estimate = constrOptim(init, fopt, grad = NULL, ui = identity(3), ci =zeros(1,3))$par
  
  #define Richards curve prediction function (function based on date)
  logcurve = Vectorize(function(date){return(ifelse(date<min(dates), NA, der_logistic(1+as.integer(date - min(dates)),
                                                                                               estimate[1], estimate[2], estimate[3])))})
  
  return(logcurve) 
}

sample_path_nhpois = function(horizon, cuml_intensity, inv_cuml_intensity){
  #sample a path with certain horizon from the inhomogeneous poisson proces with given (inverse) 
  # cumulative intensity function 
  cur_time = 0
  count = 0
  jump_times = c(0)
  counts = c(0)
  while(cur_time<horizon){
    cur_time = rnhpois(cur_time, cuml_intensity, inv_cuml_intensity)
    if(is.infinite(cur_time)){
      break
    }
    count = count + 1
    jump_times = c(jump_times, cur_time)
    counts = c(counts, count)
  }
  counts = as.data.frame(counts)
  rownames(counts) = jump_times
  return(counts)
}

sample_remaining_LOS = function( current_LOS, departure_cdf, departure_icdf){
  # this function obtains a sample from the conditional length of stay CDF, where the condition is that the length of stay (LOS)
  # is larger than "current_LOS".
  return(max(1/(24*60), departure_icdf(runif(1)*(1-departure_cdf(current_LOS-2)) + departure_cdf(current_LOS-2)) - current_LOS+2))
}


get_simple_function = function(xvals, yvals, cdf = FALSE){
  #obtain a "simple function", which returns the last observed yval at xvals
  
  #sort the data
  ordered_data = cbind(xvals,yvals)[order(xvals),]
  xvals = ordered_data[,1]
  yvals = ordered_data[,2]
  
  #return function
  return(Vectorize(function(x){
    if(x< min(xvals)){return(0)
    }else{
      if(cdf  & (x>max(xvals))){return(1)}else{
        return(yvals[which(xvals<=x)[sum(xvals<=x)]])
      }
    }
  }))
}


zero_intensity_info = function(){
  return(list(cuml_intensity_NHPP = function(t){0}, icuml_intensity_NHPP = function(t){Inf}))
}

condprob_transition = function(tibble_cond_probs, tibble_probs, LOS){
  #obtain conditional probability of being in a path corresponding to a transition between departments
  prob_transition = max(tibble_cond_probs$ligduur_01)
  if(LOS == 1){return(prob_transition)}else{
    condprob = as.numeric((prob_transition-tibble_cond_probs$ligduur_01[LOS-1])/(tibble_probs$ligduur_01[LOS-1]))
    if(is.na(condprob)){
      condprob = 0
    }
    return(condprob)}}



get_inv_simple_fn = function(xvals, yvals){
  # creates a "simple function" with jumps at xvals and height yvals at xvals.
  #sort the data
  ordered_data = cbind(xvals,yvals)[order(xvals),]
  xvals = ordered_data[,1]
  yvals = ordered_data[,2]
  
  return(Vectorize(function(p){
    if(p>max(yvals)){return(max(xvals))}else{
  return(min(xvals[which(yvals>=p)]))}}))
}


create_arrival_info = function(horizon, cuml_arrival_intensities){
  if(sum(cuml_arrival_intensities) == 0) {
    return(zero_intensity_info())
  } else {
    arrival_info = list(cuml_intensity_NHPP = approxfun(seq(0, horizon), cuml_arrival_intensities , rule = 2),
                       icuml_intensity_NHPP =approxfun(cuml_arrival_intensities, seq(0,horizon),  rule = 2))
  }
  
}

constantfn= function(val){
  #return constant function
  return(Vectorize(function(t){val}))
}

laad_output_DES <- function(naam, ziekenhuis = "etz", prefix = "") {
  map <- paste0("DES/output/", naam)
  
  if (prefix == "") {
    pattern <- paste0("^", ziekenhuis, "_2020")
  } else {
    pattern <- paste0("^", ziekenhuis, "_", prefix, "_2020")
  }
  
  # Zoek het laatste bestand (op alfabetische volgorde)
  bestand <- tail(list.files(map, pattern, full.names = TRUE), n = 1)
  
  out <- read_delim(bestand, locale = nl, delim = ";", trim_ws = TRUE)
  
  print(paste("Bestand geladen:", bestand))
  
  return(out)
}



#---------------------------------- Unit Tests ----------------------------------------

unittest_NHPP_sampling = function(){
  #check that the number of counts from an inhomogeneous poisson proces indeed follow a poisson distribution
  # with the correct parameter
  horizon = 100
  intervals = seq(0,horizon,length.out = 5)
  cuml_intensity = function(t){20/(1+exp(6-t/10))}
  inv_cuml_intensity = function(t){10*(6-log(20/t-1))}
  M = 1000
  counts_intervals = c()
  for(i in seq(M)){
    path = sample_path_nhpois(horizon, cuml_intensity , inv_cuml_intensity = inv_cuml_intensity )
    jump_times = as.numeric(rownames(path))
    counts_intervals = rbind(counts_intervals, sapply(seq(2, length(intervals)), function(rep){sum((jump_times > intervals[rep-1])&(jump_times<intervals[rep]))}))
  }
  for(i in seq(length(intervals)-1)){
    h = hist(counts_intervals[,i],length(unique(counts_intervals[,i])), probability = TRUE, main = paste('num arrivals interval [', toString(intervals[i]),' , ',toString(intervals[i+1]) ,'] vs. theor', sep = ''))
    mids = unique(round(h$mids))
    lines(mids, dpois(mids, lambda = cuml_intensity(intervals[i+1])-cuml_intensity(intervals[i])), col = 'red', lwd = 2)
  }
}

unittest_get_simple = function(){
  xvals = seq(0,9,1)
  yvals = c(0.1,0.3,0.35,0.4,0.45,0.5,0.55,0.6,0.7,1)
  f = get_simple_function(xvals, yvals)
  g = get_inv_simple_fn(xvals,yvals)
  xx = seq(-1,11,0.01)
  plot(xx, f(xx), type = 'l')
  
  yy = seq(0,1,0.01)
  plot(yy, g(yy))
}

if(sys.nframe()==0){
  
 unittest_get_simple()
  unittest_NHPP_sampling()
  }