

ewma <- compiler::cmpfun(function(x, a) {
  #ewma function
    n <- length(x)
    s <- rep(NA,n)
    s[1] <- x[1]
    if (n > 1) {
      for (i in 2:n) {
        s[i] <- a * x[i] + (1 - a) * s[i-1]
      }
    }
    return(s)
  })

vind_start_2e_golf = function(dates, counts){
  #start 2e golf voor besmettingen is (arbitrair) vastgezet op de eerste tijd dat een EWMA met alpha = 0.05 groter is can 0.5
  counts = counts[dates > as.Date('2020-07-01')]
  dates = dates[dates > as.Date('2020-07-01')]
  return(first(dates[ewma(counts, 0.05)>0.5]))
}

voorspel_lgc <- function(mod, df, horizon = 7, level_pi = .80) {
  # Haal de naam van de afhankelijke variabele op
  dep_naam <- all.vars(summary(mod)$formula)[1]
  
  # Maak een nieuw data frame
  num <- 1:(nrow(df) + horizon)
  datum_nieuw <- map_dfr(list(x = 1:horizon), ~ max(df$datum) + days(.))
  datum <- c(df$datum, datum_nieuw)
  df_pred <- tibble(datum, num)
  
  # Voeg de afhankelijke variabele toe
  dep <- c(pull(df, dep_naam), rep(NA, horizon))
  df_pred[dep_naam] <- dep
  
  # Maak voorspellingen en transformeer ze terug
  # N.B.: Ik heb hier voor predictie-intervallen gekozen, dat lijkt me in dat geval geschikter dan
  # betrouwbaarheidsintervallen. Eens?
  pred <- predFit(mod, df_pred, interval = "prediction", level = level_pi) %>%
    as_tibble()
  pred$datum <- df_pred$datum
  pred$obs <- pull(df_pred, dep_naam)
  
  return(pred)
}

voorspel_nieuwe_opnames <- function(preds) {
  out <- preds %>%
    mutate(
      nieuwe_opn = obs - lag(obs, default = 0),
      log = fit - lag(fit, default = NA),
      log_lwr = lwr - lag(fit, default = NA),
      log_upr = upr - lag(fit, default = NA),
      # De schatting kan niet lager dan 0 zijn
      log = ifelse(log < 0, 0, log),
      log_lwr = ifelse(log_lwr < 0, 0, log_lwr),
      log_upr = ifelse(log_upr < 0, 0, log_upr)
    ) %>%
    mutate_if(is_double, ~ round(., 1))
  
  return(out)
}

fit_curve_krijg_voorspellingen = function(df, horizon){
  #fit een (uigebreide) logistische curve en retourneert voorspellingen
  # df moet kolom hebben met "aantal_cumulatief" en kolom "num" wat datum nummers voorstelt
  lgc = tryCatch({
    # Schat parameters voor 4 of 5 parameter RC
    model_params <- modpar(df$num, df$aantal_cumulatief,
                           # force4par = TRUE, # Alleen parameters voor de Richards curve
                           pn.options = "myoptions", suppress.text = TRUE)
    
    
    if (is.na(model_params$RAsym)) {
       nls(aantal_cumulatief ~
                   SSposnegRichards(num,
                                    Asym = Asym, K = K, Infl = Infl, M = M,
                                    modno = 12, # Model 12 is een simpele Richards curve met 4 parameters
                                    pn.options = "myoptions"
                   ),
                 data = df, control = nls.control(minFactor = 1 / 4096 / 40)
      )
    } else {
      nls(aantal_cumulatief ~
                   SSposnegRichards(num,
                                    Asym = Asym, K = K, Infl = Infl, M = M, RAsym = RAsym,
                                    modno = 11, # Model 11 is een Richards curve met 5 parameters
                                    pn.options = "myoptions"
                   ),
                 data = df, control = nls.control(minFactor = 1 / 4096 / 40)
      )
    }
  },
  error=function(cond) {
    # In dit geval 
    # getInitial(aantal_cumulatief ~ SSlogis(num, Asym, xmid, scal), data = df)
    
    tryCatch({
      # fit een standaard logistisch model met 3 parameters
      nls(aantal_cumulatief ~ SSlogis(num, Asym, xmid, scal), 
              df)
      },error = function(cond){
                # als zelfs dit niet lukt, fit dan een exponentieel model
                # Estimate the rest parameters using a linear model
                print('exponentieel..')
                model.0 <- lm(log(aantal_cumulatief) ~ num, data=df)  
                alpha.0 <- exp(coef(model.0)[1])
                beta.0 <- coef(model.0)[2]
                start <- list(alpha = alpha.0, beta = beta.0)
                
                nls(aantal_cumulatief ~ alpha * exp(beta * num), 
                    df, start = start)
              })
    
  })
  preds_lgc <- voorspel_lgc(lgc, df, horizon = horizon)
  return(voorspel_nieuwe_opnames(preds_lgc))
}

voorspel_besmettingen_en_sla_op = function(horizon = 14, start_tweede_golf_regio = as.Date('2020-07-01'), plot_ = F, print_=F){
  if(!exists('df_besmettingen_tot')){ # read data from website
    df_besmettingen_tot = read_delim('https://data.rivm.nl/covid-19/COVID-19_aantallen_gemeente_per_dag.csv', delim = ';')
  }
  
  #df van besmettingen in ROAZ regio
  df_besmettingen_naz_west = df_besmettingen_tot%>%
                      filter(ROAZ_region == 'Netwerk Acute Zorg West')%>%
                      select(Date_of_publication, Total_reported, Municipality_name)%>%
                      rename(datum = Date_of_publication, Besmettingen = Total_reported, Gemeente = Municipality_name )
  
  #aggregeer op roaz regio
  df_besmettingen_roaz = df_besmettingen_naz_west%>%
                         group_by(datum)%>% summarise(Besmettingen = sum(Besmettingen), .groups = 'drop_last') %>% ungroup()%>%
                          filter(datum >= start_tweede_golf_regio)
  
  
  df_besmettingen_roaz$num = c(1,1+cumsum(as.integer(diff(df_besmettingen_roaz$datum))))
  df_besmettingen_roaz$aantal_cumulatief = cumsum(df_besmettingen_roaz$Besmettingen)
  
  df_pred = fit_curve_krijg_voorspellingen(df_besmettingen_roaz, horizon)
  write_csv(df_pred %>% select(datum,`Nieuwe Besmettingen`= nieuwe_opn, Ondergrens = log_lwr,Voorspelling = log,  Bovengrens = log_upr), 'predictions/voorspelde_besmettingen_naz_west.csv')
  if(plot_){print(ggplotly(ggplot(df_pred, aes(x = datum)) + geom_line(aes(y = log)) + geom_point(aes(y = nieuwe_opn)) + geom_ribbon(aes(ymin = log_lwr, ymax = log_upr), alpha = 0.2 ,fill= 'red') + ggtitle('regio')))}
  for(gemeente in unique(df_besmettingen_naz_west$Gemeente[!is.na(df_besmettingen_naz_west$Gemeente)])){
    if(print_){print(gemeente)}
    #aggregeer op gemeente niveau
    df_besmettingen_gemeente = df_besmettingen_naz_west%>%filter(Gemeente == gemeente)%>%
      group_by(datum)%>% summarise(Besmettingen = sum(Besmettingen), .groups = 'drop_last') %>% ungroup() %>% filter(datum >= start_tweede_golf_regio)
    
    #als de 2e golf veel later start voor de gemeente, vind dan de start van 2e golf voor die gemeente
    start_tweede_golf_gemeente = vind_start_2e_golf(df_besmettingen_gemeente$datum, df_besmettingen_gemeente$Besmettingen)
    df_besmettingen_gemeente = df_besmettingen_gemeente %>% filter(datum >= start_tweede_golf_gemeente)
    
    #reken datum nummers en cumulatieve besmettingen uit
    df_besmettingen_gemeente$num = c(1,1+cumsum(as.integer(diff(df_besmettingen_gemeente$datum))))
    df_besmettingen_gemeente$aantal_cumulatief = cumsum(df_besmettingen_gemeente$Besmettingen)
    
    #fit de curve, schrijf voorspellingen weg
    df_pred_gemeente = fit_curve_krijg_voorspellingen(df_besmettingen_gemeente, horizon)
    if(plot_){print(ggplotly(ggplot(df_pred_gemeente, aes(x = datum)) + geom_line(aes(y = log)) + geom_point(aes(y = nieuwe_opn)) + geom_ribbon(aes(ymin = log_lwr, ymax = log_upr), alpha = 0.2 ,fill= 'red') + ggtitle(gemeente)))}
    write_csv(df_pred_gemeente %>% select(datum,`Nieuwe Besmettingen`= nieuwe_opn,Ondergrens = log_lwr,Voorspelling = log,  Bovengrens = log_upr), paste0('predictions/voorspelde_besmettingen_',gemeente,'.csv'))
  }

}

